from __future__ import absolute_import, division, print_function


__metaclass__ = type

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


def _tmplt_ospf_address_family_cmd(config_data):
    if "address_family" in config_data:
        command = ["address-family ipv4 multicast", "exit-address-family"]
        if config_data["address_family"].get("topology"):
            if "base" in config_data["address_family"].get("topology"):
                command.insert(1, "topology base")
            elif "name" in config_data["address_family"].get("topology"):
                cmd = "topology {name}".format(**config_data["address_family"].get("topology"))
                if "tid" in config_data["address_family"].get("topology"):
                    cmd += " tid {tid}".format(**config_data["address_family"].get("topology"))
                command.insert(1, cmd)
        return command


def _tmplt_ospf_area_nssa(config_data):
    if "nssa" in config_data:
        nssa_data = config_data["nssa"]
        command = "area {area_id} nssa".format(**config_data)
        if "default_information_originate" in nssa_data:
            default_info = nssa_data["default_information_originate"]
            command += " default-information-originate"
            metric = default_info.get("metric")
            if metric is not None:
                command += " metric {metric}".format(metric=metric)
            metric_type = default_info.get("metric_type")
            if metric_type is not None:
                command += " metric-type {metric_type}".format(metric_type=metric_type)
            if default_info.get("nssa_only"):
                command += " nssa-only"
        if nssa_data.get("no_ext_capability"):
            command += " no-ext-capability"
        if nssa_data.get("no_redistribution"):
            command += " no-redistribution"
        if nssa_data.get("no_summary"):
            command += " no-summary"
        return command


def _tmplt_ospf_domain_id(config_data):
    if "domain_id" in config_data:
        command = "domain-id"
        if "ip_address" in config_data["domain_id"]:
            if "address" in config_data["domain_id"]["ip_address"]:
                command += " {address}".format(**config_data["domain_id"]["ip_address"])
                if "secondary" in config_data["domain_id"]["ip_address"]:
                    command += " secondary".format(**config_data["domain_id"]["ip_address"])
        elif "null" in config_data["domain_id"]:
            command += " null"
        return command


def _tmplt_ospf_max_metric(config_data):
    if "max_metric" in config_data:
        command = "max-metric"
        if "router_lsa" in config_data["max_metric"]:
            command += " router-lsa"
        if "include_stub" in config_data["max_metric"]:
            command += " include-stub"
        if "set_summary_lsa" in config_data["max_metric"]:
            command += " summary-lsa"
        if "summary_lsa" in config_data["max_metric"]:
            command += " summary-lsa {summary_lsa}".format(**config_data["max_metric"])
        if "set_external_lsa" in config_data["max_metric"]:
            command += " external-lsa"
        if "external_lsa" in config_data["max_metric"]:
            command += " external-lsa {external_lsa}".format(**config_data["max_metric"])
        if "on_startup" in config_data["max_metric"]:
            if "time" in config_data["max_metric"]["on_startup"]:
                command += " on-startup {time}".format(**config_data["max_metric"]["on_startup"])
            elif "wait_for_bgp" in config_data["max_metric"]["on_startup"]:
                command += " on-startup wait-for-bgp"
        return command


def _tmplt_ospf_mpls_traffic_eng(config_data):
    if "traffic_eng" in config_data["mpls"]:
        command = "mpls traffic-eng"
        if "area" in config_data["mpls"]["traffic_eng"]:
            command += " area {area}".format(**config_data["mpls"]["traffic_eng"])
        elif "autoroute_exclude" in config_data["mpls"]["traffic_eng"]:
            command += " autoroute-exclude prefix-list {autoroute_exclude}".format(
                **config_data["mpls"]["traffic_eng"],
            )
        elif "interface" in config_data["mpls"]["traffic_eng"]:
            command += " interface {int_type}".format(
                **config_data["mpls"]["traffic_eng"]["interface"],
            )
            if "area" in config_data["mpls"]["traffic_eng"]["interface"]:
                command += " area {area}".format(**config_data["mpls"]["traffic_eng"]["interface"])
        elif "mesh_group" in config_data["mpls"]["traffic_eng"]:
            command += " mesh-group {id} {interface}".format(
                **config_data["mpls"]["traffic_eng"]["mesh_group"],
            )
            if "area" in config_data["mpls"]["traffic_eng"]["mesh_group"]:
                command += " area {area}".format(**config_data["mpls"]["traffic_eng"]["mesh_group"])
        elif "multicast_intact" in config_data["mpls"]["traffic_eng"]:
            command += " multicast-intact"
        elif "router_id_interface" in config_data["mpls"]["traffic_eng"]:
            command += " router-id {router_id_interface}".format(
                **config_data["mpls"]["traffic_eng"],
            )
        return command


def _tmplt_ospf_neighbor(config_data):
    if "neighbor" in config_data:
        command = "neighbor"
        if "address" in config_data["neighbor"]:
            command += " {address}".format(**config_data["neighbor"])
        if "cost" in config_data["neighbor"]:
            command += " cost {cost}".format(**config_data["neighbor"])
        if "database_filter" in config_data["neighbor"]:
            command += " database-filter all out"
        if "poll_interval" in config_data["neighbor"]:
            command += " poll-interval {poll_interval}".format(**config_data["neighbor"])
        if "priority" in config_data["neighbor"]:
            command += " priority {priority}".format(**config_data["neighbor"])
        return command


def _tmplt_ospf_passive_interfaces(config_data):
    cmd = ""  # Initialize cmd to avoid potential unbound error
    if "passive_interfaces" in config_data:
        if config_data["passive_interfaces"].get("interface"):
            if config_data["passive_interfaces"].get("set_interface"):
                for each in config_data["passive_interfaces"]["interface"]:
                    cmd = "passive-interface {0}".format(each)
            elif not config_data["passive_interfaces"].get("set_interface"):
                for each in config_data["passive_interfaces"]["interface"]:
                    cmd = "no passive-interface {0}".format(each)
        return cmd


class Ospfv2Template(NetworkTemplate):
    def __init__(self, lines=None):
        super(Ospfv2Template, self).__init__(lines=lines, tmplt=self)

    PARSERS = [
        {
            "name": "pid",
            "getval": re.compile(
                r"""
                ^router\sospf
                (\s(?P<pid>\d+))
                (\s(vrf\s(?P<vrf_value>\S+)))?
                $""",
                re.VERBOSE,
            ),
            "setval": "router ospf {{ process_id }}"
            "{{ (' vrf ' + vrf ) if vrf is defined else '' }}",
            "result": {
                "processes": {
                    "{{ pid }}": {"process_id": "{{ pid|int }}", "vrf": "{{ vrf_value }}"},
                },
            },
            "shared": True,
        },
        {
            "name": "adjacency",
            "getval": re.compile(
                r"""
                \sadjacency\sstagger
                (\s(?P<min>\d+))?
                (\s(?P<none>none))?
                (\s(?P<max>\d+))?
                $""",
                re.VERBOSE,
            ),
            "setval": "adjacency stagger {{ 'none' if adjacency.none else adjacency.min_adjacency }} {{ adjacency.max_adjacency }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "adjacency": {
                            "min_adjacency": "{{ min|int }}",
                            "max_adjacency": "{{ max|int }}",
                            "none": "{{ True if none_adj is defined else False }}",
                        },
                    },
                },
            },
        },
        {
            "name": "address_family",
            "getval": re.compile(
                r"""
                \stopology
                (\s(?P<base>base))?
                (\s(?P<name>\S+))?
                (\stid\s(?P<tid>\S+))?
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_address_family_cmd,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "address_family": {
                            "topology": {
                                "base": "{{ True if base is defined }}",
                                "name": "{{ name }}",
                                "tid": "{{ tid }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "authentication",
            "getval": re.compile(
                r"""
                \sarea
                (\s(?P<area_id>\S+))?
                (\s(?P<auth>authentication))?
                (\s(?P<md>message-digest))?
                $""",
                re.VERBOSE,
            ),
            "setval": "area {{ area_id }} authentication"
            "{{ ' message-digest' if authentication.message_digest else '' }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "authentication": {
                                    "enable": "{{ True if auth is defined and md is undefined }}",
                                    "message_digest": "{{ not not md }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "capability",
            "getval": re.compile(
                r"""
                \sarea
                (\s(?P<area_id>\S+))?
                (\s(?P<capability>capability))?
                \sdefault-exclusion
                $""",
                re.VERBOSE,
            ),
            "setval": "area {{ area_id }} capability default-exclusion",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "capability": "{{ not not capability }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "default_cost",
            "getval": re.compile(
                r"""
                \sarea
                (\s(?P<area_id>\S+))?
                \sdefault-cost
                (\s(?P<default_cost>\S+))?
                $""",
                re.VERBOSE,
            ),
            "setval": "area {{ area_id }} default-cost {{ default_cost }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "default_cost": "{{ default_cost|int }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "filter_list",
            "getval": re.compile(
                r"""
                \s+area
                (\s(?P<area_id>\S+))?
                \sfilter-list\sprefix
                (\s(?P<name>\S+))?
                (\s(?P<dir>\S+))?
                $""",
                re.VERBOSE,
            ),
            "setval": "area {{ area_id }}" " filter-list prefix {{ name }} {{ direction }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "filter_list": [
                                    {
                                        "name": "{{ name }}",
                                        "direction": "{{ dir }}",
                                    },
                                ],
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "nssa",
            "getval": re.compile(
                r"""
                \sarea
                (\s(?P<area_id>\S+))?
                (\s(?P<nssa>nssa))?
                (\s(?P<no_redis>no-redistribution))?
                (\s(?P<def_origin>default-information-originate))?
                (\smetric\s(?P<metric>\d+))?
                (\smetric-type\s(?P<metric_type>\d+))?
                (\s(?P<no_summary>no-summary))?
                (\s(?P<nssa_only>nssa-only))?
                (\s(?P<no_ext>no-ext-capability))?
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_area_nssa,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "nssa": {
                                    "set": "{{ True if def_origin is undefined and "
                                    "no_ext is undefined and no_redis is undefined and nssa_only is undefined }}",
                                    "default_information_originate": {
                                        "metric": "{{ metric|int }}",
                                        "metric_type": "{{ metric_type|int }}",
                                        "nssa_only": "{{ True if nssa_only is defined }}",
                                    },
                                    "no_ext_capability": "{{ True if no_ext is defined }}",
                                    "no_redistribution": "{{ True if no_redis is defined }}",
                                    "no_summary": "{{ True if no_summary is defined }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "nssa.translate",
            "getval": re.compile(
                r"""
                \sarea
                (\s(?P<area_id>\S+))?
                \snssa
                \stranslate\stype7
                (\s(?P<translate_always>always|suppress-fa))?
                $""",
                re.VERBOSE,
            ),
            "setval": "area {{ area_id }} nssa "
            "translate type7 {{ nssa.translate if nssa.translate is defined }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "nssa": {
                                    "translate": "{{ translate_always }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "ranges",
            "getval": re.compile(
                r"""
                \sarea
                (\s(?P<area_id>\S+))
                \srange
                (\s(?P<address>\S+))
                (\s(?P<netmask>\S+))
                (\s(?P<advertise>advertise))?
                (\s(?P<not_advertise>not-advertise))?
                (\s(cost\s(?P<cost>\d+)))
                $""",
                re.VERBOSE,
            ),
            "setval": "area {{ area_id }} range {{ address }} {{ netmask }}"
            "{{ (' advertise') if advertise is defined and advertise else '' }}"
            "{{ (' not-advertise') if not_advertise is defined and not_advertise else '' }}"
            "{{ (' cost ' + cost|string) if cost is defined else '' }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "ranges": [
                                    {
                                        "address": "{{ address }}",
                                        "netmask": "{{ netmask }}",
                                        "advertise": "{{ True if advertise is defined }}",
                                        "cost": "{{ cost }}",
                                        "not_advertise": "{{ True if not_advertise is defined }}",
                                    },
                                ],
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "sham_link",
            "getval": re.compile(
                r"""
                \sarea
                (\s(?P<area_id>\S+))?
                \ssham-link
                (\s(?P<source>\S+))?
                (\s(?P<destination>\S+))?
                (\s(cost\s(?P<cost>\d+)))?
                (\s(ttl-security\shops\s(?P<ttl_security>\d+)))?
                $""",
                re.VERBOSE,
            ),
            "setval": "area {{ area_id }} sham-link {{ sham_link.source }} {{ sham_link.destination }}"
            "{{ (' cost ' + sham_link.cost|string) if sham_link.cost is defined else '' }}"
            "{{ (' ttl-security hops ' + sham_link.ttl_security|string) if sham_link.ttl_security is defined else '' }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "sham_link": {
                                    "source": "{{ source }}",
                                    "destination": "{{ destination }}",
                                    "cost": "{{ cost }}",
                                    "ttl_security": "{{ ttl_security }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "stub",
            "getval": re.compile(
                r"""
                \sarea
                (\s(?P<area_id>\S+))?
                (\s(?P<stub>stub))?
                (\s(?P<no_ext>no-ext-capability))?
                (\s(?P<no_sum>no-summary))?
                $""",
                re.VERBOSE,
            ),
            "setval": "area {{ area_id }} stub"
            "{{ (' no-ext-capability') if stub.no_ext_capability else ''}}"
            "{{ (' no-summary') if stub.no_summary else ''}}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "stub": {
                                    "set": "{{ True if stub is defined and no_ext is undefined and no_sum is undefined }}",
                                    "no_ext_capability": "{{ True if no_ext is defined }}",
                                    "no_summary": "{{ True if no_sum is defined }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "auto_cost",
            "getval": re.compile(
                r"""
                (\s(?P<auto_cost>auto-cost))
                (\sreference-bandwidth\s(?P<ref_band>\d+))?
                $""",
                re.VERBOSE,
            ),
            "setval": "auto-cost"
            "{{ ' reference-bandwidth ' + auto_cost.reference_bandwidth|string if auto_cost.reference_bandwidth is defined }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "auto_cost": {
                            "set": True,
                            "reference_bandwidth": "{{ ref_band }}",
                        },
                    },
                },
            },
        },
        {
            "name": "bfd",
            "getval": re.compile(
                r"""
                \sbfd
                (\s(?P<bfd>all-interfaces))?
                $""",
                re.VERBOSE,
            ),
            "setval": "bfd all-interfaces",
            "result": {"processes": {"{{ pid }}": {"bfd": "{{ True if bfd is defined }}"}}},
        },
        {
            "name": "capability.lls",
            "getval": re.compile(
                r"""
                \scapability
                (\s(?P<lls>lls))
                $""",
                re.VERBOSE,
            ),
            "setval": "capability lls",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "capability": {
                            "lls": True,
                        },
                    },
                },
            },
        },
        {
            "name": "capability.opaque",
            "getval": re.compile(
                r"""
                \scapability
                (\s(?P<opaque>opaque))
                $""",
                re.VERBOSE,
            ),
            "setval": "capability opaque",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "capability": {
                            "opaque": True,
                        },
                    },
                },
            },
        },
        {
            "name": "capability.transit",
            "getval": re.compile(
                r"""
                \scapability
                (\s(?P<transit>transit))
                $""",
                re.VERBOSE,
            ),
            "setval": "capability transit",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "capability": {
                            "transit": True,
                        },
                    },
                },
            },
        },
        {
            "name": "capability.vrf_lite",
            "getval": re.compile(
                r"""
                \scapability
                (\s(?P<vrf_lite>vrf-lite))
                $""",
                re.VERBOSE,
            ),
            "setval": "capability vrf-lite",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "capability": {
                            "vrf_lite": True,
                        },
                    },
                },
            },
        },
        {
            "name": "compatible",
            "getval": re.compile(
                r"""
                \scompatible
                (\s(?P<rfc>rfc1583|rfc1587|rfc5243))?
                $""",
                re.VERBOSE,
            ),
            "setval": "compatible"
            "{{ (' rfc1583') if compatible.rfc1583 else ''}}"
            "{{ (' rfc1587') if compatible.rfc1587 else ''}}"
            "{{ (' rfc5243') if compatible.rfc5243 else ''}}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "compatible": {
                            "rfc1583": "{{ True if 'rfc1583' in rfc else False }}",
                            "rfc1587": "{{ True if 'rfc1587' in rfc else False }}",
                            "rfc5243": "{{ True if 'rfc5243' in rfc else False }}",
                        },
                    },
                },
            },
        },
        {
            "name": "default_information",
            "getval": re.compile(
                r"""
                \sdefault-information
                (\s(?P<originate>originate))?
                (\s(?P<always>always))?
                (\smetric\s(?P<metric>\d+))?
                (\smetric-type\s(?P<metric_type>\d+))?
                (\sroute-map\s(?P<route_map>\S+))?
                $""",
                re.VERBOSE,
            ),
            "setval": "default-information"
            "{{ ' originate' if default_information.originate is defined else ''}}"
            "{{ ' always' if default_information.always is defined else '' }}"
            "{{ (' metric ' + default_information.metric|string) if default_information.metric is defined else '' }}"
            "{{ (' metric-type ' + default_information.metric_type|string) if default_information.metric_type is defined else '' }}"
            "{{ ' route-map ' + default_information.route_map if default_information.route_map is defined and default_information.metric is defined else '' }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "default_information": {
                            "originate": "{{ True if originate is defined }}",
                            "always": "{{ True if always is defined }}",
                            "metric": "{{ metric }}",
                            "metric_type": "{{ metric_type }}",
                            "route_map": "{{ route_map }}",
                        },
                    },
                },
            },
        },
        {
            "name": "default_metric",
            "getval": re.compile(
                r"""
                \sdefault-metric
                \s(?P<default_metric>\d+)?
                $""",
                re.VERBOSE,
            ),
            "setval": "default-metric {{ default_metric|string }}",
            "result": {"processes": {"{{ pid }}": {"default_metric": "{{ default_metric }}"}}},
        },
        {
            "name": "discard_route",
            "getval": re.compile(
                r"""
                (\s(?P<discard_route>discard-route))
                (\sexternal\s(?P<external>\d+))?
                (\sinternal\s(?P<internal>\d+))?
                $""",
                re.VERBOSE,
            ),
            "setval": "discard-route"
            "{{ ' external ' + discard_route.external|string if discard_route.external is defined else '' }}"
            "{{ ' internal ' + discard_route.internal|string if discard_route.internal is defined else '' }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "discard_route": {
                            "set": True,
                            "external": "{{ external }}",
                            "internal": "{{ internal }}",
                        },
                    },
                },
            },
        },
        {
            "name": "distance.admin_distance",
            "getval": re.compile(
                r"""
                \sdistance
                (\s(?P<admin_dist>\d+))
                (\s(?P<source>\S+))?
                (\s(?P<wildcard>\S+))?
                (\s(?P<acl>\S+))?
                $""",
                re.VERBOSE,
            ),
            "setval": "distance {{ distance.admin_distance.distance }} "
            "{{ ( distance.admin_distance.address + ' ' + distance.admin_distance.wildcard_bits ) if distance.admin_distance.address is defined else '' }}"
            "{{ ' ' + distance.admin_distance.acl if distance.admin_distance.acl is defined else '' }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "distance": {
                            "admin_distance": {
                                "distance": "{{ admin_dist }}",
                                "address": "{{ source }}",
                                "wildcard_bits": "{{ wildcard }}",
                                "acl": "{{ acl }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "distance.ospf",
            "getval": re.compile(
                r"""
                \sdistance\sospf
                (\sintra-area\s(?P<intra>\d+))?
                (\sinter-area\s(?P<inter>\d+))?
                (\sexternal\s(?P<external>\d+))?
                $""",
                re.VERBOSE,
            ),
            "setval": "distance ospf"
            "{{ ' inter-area ' + distance.ospf.inter_area|string if distance.ospf.inter_area is defined else '' }}"
            "{{ ' intra-area ' + distance.ospf.intra_area|string if distance.ospf.intra_area is defined else '' }}"
            "{{ ' external ' + distance.ospf.external|string if distance.ospf.external is defined else '' }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "distance": {
                            "ospf": {
                                "inter_area": "{{ inter|int }}",
                                "intra_area": "{{ intra|int }}",
                                "external": "{{ external|int }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "distribute_list.acls",
            "getval": re.compile(
                r"""
                \sdistribute-list
                (\s(?P<name>\S+))
                (\s(?P<dir>\S+))
                (\s(?P<int_pro>\S+\s\d+))?
                $""",
                re.VERBOSE,
            ),
            "setval": "distribute-list {{ name }} {{ direction }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "distribute_list": {
                            "acls": [
                                {
                                    "name": "{{ name }}",
                                    "direction": "{{ dir }}",
                                    "interface": '{{ int_pro if dir == "in" }}',
                                    "protocol": '{{ int_pro if dir == "out" }}',
                                },
                            ],
                        },
                    },
                },
            },
        },
        {
            "name": "distribute_list.prefix",
            "getval": re.compile(
                r"""
                \sdistribute-list
                (\sprefix\s(?P<prefix>\S+))
                (\sgateway\s(?P<gateway>\S+))?
                (\s(?P<dir>\S+))?
                (\s(?P<int_pro>\S+\s\S+))?
                $""",
                re.VERBOSE,
            ),
            "setval": "distribute-list prefix {{ distribute_list.prefix.name }}"
            "{{ ' gateway ' + distribute_list.prefix.gateway_name if distribute_list.prefix.gateway_name is defined else '' }}"
            "{{ ' ' + distribute_list.prefix.direction if distribute_list.prefix.direction is defined else '' }}"
            "{{ ' ' + distribute_list.prefix.interface if distribute_list.prefix.interface is defined else '' }}"
            "{{ ' ' + distribute_list.prefix.protocol if distribute_list.prefix.protocol is defined else '' }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "distribute_list": {
                            "prefix": {
                                "name": "{{ prefix }}",
                                "gateway_name": "{{ gateway }}",
                                "direction": "{{ dir if gateway is undefined }}",
                                "interface": '{{ int_pro if dir == "in" }}',
                                "protocol": '{{ int_pro if dir == "out" }}',
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "distribute_list.route_map",
            "getval": re.compile(
                r"""
                \sdistribute-list
                (\sroute-map\s(?P<route_map>\S+))
                (\s(?P<dir>\S+))
                $""",
                re.VERBOSE,
            ),
            "setval": "distribute-list route-map {{ distribute_list.route_map.name }} in",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "distribute_list": {"route_map": {"name": "{{ route_map }}"}},
                    },
                },
            },
        },
        {
            "name": "domain_id",
            "getval": re.compile(
                r"""
                \sdomain-id
                (\s(?P<address>\S+))
                (\s(?P<secondary>secondary))?
                (\s(?P<null>null))?
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_domain_id,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "domain_id": {
                            "ip_address": {
                                "address": "{{ address }}",
                                "secondary": "{{ True if secondary is defined }}",
                            },
                            "null": "{{ True if null is defined }}",
                        },
                    },
                },
            },
        },
        {
            "name": "domain_tag",
            "getval": re.compile(
                r"""
                \sdomain-tag
                (\s(?P<tag>\d+))
                $""",
                re.VERBOSE,
            ),
            "setval": "domain-tag {{ domain_tag }}",
            "result": {"processes": {"{{ pid }}": {"domain_tag": "{{ tag|int }}"}}},
        },
        {
            "name": "event_log",
            "getval": re.compile(
                r"""
                (\s(?P<event_log>event-log))?
                (\s(?P<one_shot>one-shot))?
                (\s(?P<pause>pause))?
                (\ssize\s(?P<size>\d+))?
                $""",
                re.VERBOSE,
            ),
            "setval": "event-log"
            "{{ ' one-shot' if event_log.one_shot else '' }}"
            "{{ ' pause' if event_log.pause else '' }}"
            "{{ ' size ' + event_log.size|string if event_log.size is defined else '' }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "event_log": {
                            "enable": "{{ True if event_log is defined and one_shot is undefined and pause is undefined and size is undefined }}",
                            "one_shot": "{{ True if one_shot is defined }}",
                            "pause": "{{ True if pause is defined }}",
                            "size": "{{ size }}",
                        },
                    },
                },
            },
        },
        {
            "name": "help",
            "getval": re.compile(
                r"""
                \s(?P<help>help)
                $""",
                re.VERBOSE,
            ),
            "setval": "help",
            "result": {"processes": {"{{ pid }}": {"help": "{{ True if help is defined }}"}}},
        },
        {
            "name": "ignore",
            "getval": re.compile(
                r"""
                \s(?P<ignore>ignore)
                $""",
                re.VERBOSE,
            ),
            "setval": "ignore lsa mospf",
            "result": {"processes": {"{{ pid }}": {"ignore": "{{ True if ignore is defined }}"}}},
        },
        {
            "name": "interface_id",
            "getval": re.compile(
                r"""
                (\s(?P<interface_id>interface-id\ssnmp-if-index))?
                $""",
                re.VERBOSE,
            ),
            "setval": "interface-id snmp-if-index",
            "result": {
                "processes": {
                    "{{ pid }}": {"interface_id": "{{ True if interface_id is defined }}"},
                },
            },
        },
        {
            "name": "ispf",
            "getval": re.compile(
                r"""
                \s(?P<ispf>ispf)
                $""",
                re.VERBOSE,
            ),
            "setval": "ispf",
            "result": {"processes": {"{{ pid }}": {"ispf": "{{ True if ispf is defined }}"}}},
        },
        {
            "name": "limit",
            "getval": re.compile(
                r"""
                \slimit\sretransmissions
                (\sdc\s(?P<dc_num>\d+))?
                (\sdc\sdisable(?P<dc_disable>))?
                (\snon-dc\s(?P<non_dc_num>\d+))?
                (\snon-dc\sdisable(?P<non_dc_disable>))?
                $""",
                re.VERBOSE,
            ),
            "setval": "limit retransmissions"
            "{{ ' dc ' + limit.dc.number|string if limit.dc.number is defined }}"
            "{{ ' dc disable' if limit.dc.disable and limit.dc.number is not defined }}"
            "{{ ' non-dc ' + limit.non_dc.number|string if limit.non_dc.number is defined }}"
            "{{ ' non-dc disable' if limit.non_dc.disable and limit.non_dc.number is not defined }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "limit": {
                            "dc": {
                                "number": "{{ dc_num|int }}",
                                "disable": "{{ True if dc_disable is defined }}",
                            },
                            "non_dc": {
                                "number": "{{ non_dc_num|int }}",
                                "disable": "{{ True if non_dc_disable is defined }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "local_rib_criteria",
            "getval": re.compile(
                r"""
                (\s(?P<local>local-rib-criteria))?
                (\s(?P<forward>forwarding-address))?
                (\s(?P<inter>inter-area-summary))?
                (\s(?P<nssa>nssa-translation))?
                $""",
                re.VERBOSE,
            ),
            "setval": "local-rib-criteria"
            "{{ ' forwarding-address' if local_rib_criteria.forwarding_address else '' }}"
            "{{ ' inter-area-summary' if local_rib_criteria.inter_area_summary else '' }}"
            "{{ ' nssa-translation' if local_rib_criteria.nssa_translation else '' }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "local_rib_criteria": {
                            "enable": "{{ True if local is defined and forward is undefined and inter is undefined and nssa is undefined }}",
                            "forwarding_address": "{{ True if forward is defined }}",
                            "inter_area_summary": "{{ True if inter is defined }}",
                            "nssa_translation": "{{ True if nssa is defined }}",
                        },
                    },
                },
            },
        },
        {
            "name": "log_adjacency_changes",
            "getval": re.compile(
                r"""
                (\s(?P<log>log-adjacency-changes))?
                (\s(?P<detail>detail))?
                $""",
                re.VERBOSE,
            ),
            "setval": "log-adjacency-changes"
            "{{ ' detail' if log_adjacency_changes.detail else '' }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "log_adjacency_changes": {
                            "set": "{{ True if log is defined and detail is undefined }}",
                            "detail": "{{ True if detail is defined }}",
                        },
                    },
                },
            },
        },
        {
            "name": "max_lsa",
            "getval": re.compile(
                r"""\smax-lsa
                    (\s(?P<number>\d+))?
                    (\s(?P<threshold>\d+))?
                    (\s(?P<warning>warning-only))?
                    (\signore-count\s(?P<ignore_count>\d+))?
                    (\signore-time\s(?P<ignore_time>\d+))?
                    (\sreset-time\s(?P<reset_time>\d+))?
                    $""",
                re.VERBOSE,
            ),
            "setval": "max-lsa {{ max_lsa.number|string }}"
            "{{ ' ' + max_lsa.threshold_value|string if max_lsa.threshold_value is defined else '' }}"
            "{{ ' warning-only' if max_lsa.warning_only else '' }}"
            "{{ ' ignore-count ' + max_lsa.ignore_count|string if max_lsa.ignore_count is defined and not max_lsa.warning_only else '' }}"
            "{{ ' ignore-time ' + max_lsa.ignore_time|string if max_lsa.ignore_time is defined and not max_lsa.warning_only else '' }}"
            "{{ ' reset-time ' + max_lsa.reset_time|string if max_lsa.reset_time is defined and not max_lsa.warning_only else '' }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "max_lsa": {
                            "number": "{{ number }}",
                            "threshold_value": "{{ threshold }}",
                            "ignore_count": "{{ ignore_count }}",
                            "ignore_time": "{{ ignore_time }}",
                            "reset_time": "{{ reset_time }}",
                            "warning_only": "{{ True if warning is defined }}",
                        },
                    },
                },
            },
        },
        {
            "name": "max_metric",
            "getval": re.compile(
                r"""
                \smax-metric
                (\s(?P<router_lsa>router-lsa))?
                (\s(?P<include_stub>include-stub))?
                (\s(?P<summary_lsa_bool>summary-lsa))?
                (\ssummary-lsa\s(?P<summary_lsa>\d+))?
                (\s(?P<external_lsa_bool>external-lsa))?
                (\sexternal-lsa\s(?P<external_lsa>\d+))?
                (\son-startup\s(?P<startup_time>\d+))?
                (\son-startup\s(?P<startup_wait>\S+))?
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_max_metric,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "max_metric": {
                            "router_lsa": "{{ True if router_lsa is defined }}",
                            "external_lsa": "{{ external_lsa }}",
                            "set_summary_lsa": "{{ not not summary_lsa_bool }}",
                            "set_external_lsa": "{{ not not external_lsa_bool }}",
                            "include_stub": "{{ not not include_stub }}",
                            "on_startup": {
                                "time": "{{ startup_time }}",
                                "wait_for_bgp": "{{ True if startup_wait is defined }}",
                            },
                            "summary_lsa": "{{ summary_lsa }}",
                        },
                    },
                },
            },
        },
        {
            "name": "maximum_paths",
            "getval": re.compile(
                r"""
                \smaximum-paths
                \s(?P<paths>\d+)
                $""",
                re.VERBOSE,
            ),
            "setval": "maximum-paths {{ maximum_paths|string }}",
            "result": {"processes": {"{{ pid }}": {"maximum_paths": "{{ paths }}"}}},
        },
        {
            "name": "mpls.ldp.autoconfig",
            "getval": re.compile(
                r"""
                \smpls\sldp
                (\s(?P<autoconfig>autoconfig))
                (\sarea\s(?P<area>\S+))?
                $""",
                re.VERBOSE,
            ),
            "setval": "mpls ldp autoconfig"
            "{{ ' area ' + mpls.ldp.autoconfig.area if mpls.ldp.autoconfig.area is defined }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "mpls": {
                            "ldp": {
                                "autoconfig": {
                                    "set": "{{ True if autoconfig is defined and area is undefined }}",
                                    "area": "{{ area }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mpls.ldp.sync",
            "getval": re.compile(
                r"""
                \smpls\sldp
                (\s(?P<sync>sync))
                $""",
                re.VERBOSE,
            ),
            "setval": "mpls ldp sync",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "mpls": {
                            "ldp": {
                                "sync": True,
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mpls.traffic_eng",
            "getval": re.compile(
                r"""\s+mpls
                    \straffic-eng*
                    \s*(?P<area>area\s\S+)*
                    \s*(?P<autoroute>autoroute-exclude\s\S+\s\S+)*
                    \s*(?P<interface>interface\s(?P<int_type>\S+\s\S+)\s(?P<int_area>area\s\S+))*
                    \s*(?P<mesh>mesh-group\s\d+\s(?P<mesh_int>\S+\s\S+)\s(?P<mesh_area>area\s\d+))*
                    \s*(?P<multicast>multicast-intact)*
                    \s*(?P<router>router-id\s(?P<router_int>\S+\s\S+))
                    *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_mpls_traffic_eng,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "mpls": {
                            "traffic_eng": {
                                "area": "{{ area.split(" ")[1] }}",
                                "autoroute_exclude": "{{ autoroute.split(" ")[2] }}",
                                "interface": {
                                    "interface_type": "{{ int_type }}",
                                    "area": "{{ int_area.split(" ")[1] }}",
                                },
                                "mesh_group": {
                                    "id": "{{ mesh.split(" ")[1] }}",
                                    "interface": "{{ mest_int }}",
                                    "area": "{{ mesh_area.split(" ")[1] }}",
                                },
                                "multicast_intact": "{{ True if multicast is defined }}",
                                "router_id_interface": "{{ router_int }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor",
            "getval": re.compile(
                r"""
                \sneighbor
                (\s(?P<address>\S+))
                (\scost\s(?P<cost>\d+))
                (\sdatabase-filter\sall\sout\s(?P<db_filter>))?
                (\spoll-interval\s(?P<poll>\d+))?
                (\spriority\s(?P<priority>\d+))?
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_neighbor,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "neighbor": {
                            "address": "{{ address }}",
                            "cost": "{{ cost }}",
                            "database_filter": "{{ True if db_filter is defined }}",
                            "poll_interval": "{{ poll }}",
                            "priority": "{{ priority }}",
                        },
                    },
                },
            },
        },
        {
            "name": "network",
            "getval": re.compile(
                r"""
                \snetwork
                (\s(?P<address>\S+))?
                (\s(?P<wildcard>\S+))?
                (\sarea\s(?P<area>\S+))?
                $""",
                re.VERBOSE,
            ),
            "setval": "network"
            "{{ (' ' + address + ' ' + wildcard_bits) if address is defined }}"
            "{{ ' area ' + area if area is defined }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "network": [
                            {
                                "address": "{{ address }}",
                                "wildcard_bits": "{{ wildcard }}",
                                "area": "{{ area }}",
                            },
                        ],
                    },
                },
            },
        },
        {
            "name": "nsf.cisco",
            "getval": re.compile(
                r"""
                \snsf
                (\s(?P<cisco>cisco))
                (\s(?P<helper>helper))?
                (\s(?P<disable>disable))?
                $""",
                re.VERBOSE,
            ),
            "setval": "nsf cisco helper" "{{ ' disable' if nsf.cisco.disable }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "nsf": {
                            "cisco": {
                                "helper": "{{ True if helper is defined }}",
                                "disable": "{{ True if disable is defined }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "nsf.ietf.disable",
            "getval": re.compile(
                r"""
                \snsf
                (\s(?P<ietf>ietf))
                (\s(?P<helper>helper))
                (\s(?P<disable>disable))
                $""",
                re.VERBOSE,
            ),
            "setval": "nsf ietf helper disable",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "nsf": {
                            "ietf": {
                                "helper": True,
                                "disable": True,
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "nsf.ietf.strict_lsa_checking",
            "getval": re.compile(
                r"""
                \snsf
                (\s(?P<ietf>ietf))
                (\s(?P<helper>helper))
                (\s(?P<strict>strict-lsa-checking))
                $""",
                re.VERBOSE,
            ),
            "setval": "nsf ietf helper strict-lsa-checking",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "nsf": {
                            "ietf": {
                                "helper": "{{ True  }}",
                                "strict_lsa_checking": True,
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "passive_interfaces.default",
            "getval": re.compile(
                r"""
                \spassive-interface
                \s(?P<default_value>default)
                $""",
                re.VERBOSE,
            ),
            "setval": "passive-interface" "{{ ' default' if passive_interfaces.default }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "passive_interfaces": {
                            "default": "{{ True if default_value is defined }}",
                        },
                    },
                },
            },
        },
        {
            "name": "passive_interfaces.interface",
            "getval": re.compile(
                r"""
                (\s(?P<no>no))?
                \spassive-interface
                \s(?P<interface>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_passive_interfaces,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "passive_interfaces": {
                            "interface": {
                                "set_interface": "{{ False if no is defined else True }}",
                                "name": ["{{ interface if 'default' not in interface }}"],
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "prefix_suppression",
            "getval": re.compile(
                r"""
                \s(?P<prefix_sup>prefix-suppression)
                $""",
                re.VERBOSE,
            ),
            "setval": "prefix-suppression",
            "result": {
                "processes": {
                    "{{ pid }}": {"prefix_suppression": "{{ True if prefix_sup is defined }}"},
                },
            },
        },
        {
            "name": "priority",
            "getval": re.compile(
                r"""
                \spriority
                \s(?P<priority>\d+)
                $""",
                re.VERBOSE,
            ),
            "setval": "priority {{ priority }}",
            "result": {"processes": {"{{ pid }}": {"priority": "{{ priority }}"}}},
        },
        {
            "name": "queue_depth.hello.max_packets",
            "getval": re.compile(
                r"""
                \squeue-depth\shello
                (\s(?P<max_packets>\d+))
                $""",
                re.VERBOSE,
            ),
            "setval": "queue-depth hello " "{{ queue_depth.hello.max_packets|string }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "queue_depth": {
                            "hello": {
                                "max_packets": "{{ max_packets }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "queue_depth.hello.unlimited",
            "getval": re.compile(
                r"""
                \squeue-depth\shello
                (\s(?P<unlimited>unlimited))
                $""",
                re.VERBOSE,
            ),
            "setval": "queue-depth hello unlimited",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "queue_depth": {
                            "hello": {
                                "unlimited": True,
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "queue_depth.update.max_packets",
            "getval": re.compile(
                r"""
                \squeue-depth\supdate
                (\s(?P<max_packets>\d+))?
                $""",
                re.VERBOSE,
            ),
            "setval": "queue-depth update " "{{ queue_depth['update'].max_packets|string }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "queue_depth": {
                            "update": {
                                "max_packets": "{{ max_packets }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "queue_depth.update.unlimited",
            "getval": re.compile(
                r"""
                \squeue-depth\supdate
                (\s(?P<unlimited>unlimited))
                $""",
                re.VERBOSE,
            ),
            "setval": "queue-depth update unlimited",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "queue_depth": {
                            "update": {
                                "unlimited": True,
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "router_id",
            "getval": re.compile(
                r"""
                \srouter-id
                (\s(?P<id>\S+))?
                $""",
                re.VERBOSE,
            ),
            "setval": "router-id {{ router_id }}",
            "result": {"processes": {"{{ pid }}": {"router_id": "{{ id }}"}}},
        },
        {
            "name": "shutdown",
            "getval": re.compile(
                r"""
                \s(?P<shutdown>shutdown)
                $""",
                re.VERBOSE,
            ),
            "setval": "shutdown",
            "result": {
                "processes": {"{{ pid }}": {"shutdown": "{{ True if shutdown is defined }}"}},
            },
        },
        {
            "name": "summary_address.not_advertise",
            "getval": re.compile(
                r"""
                \ssummary-address
                (\s(?P<address>\S+))
                (\s(?P<mask>\S+))
                (\s(?P<not_adv>not-advertise))
                $""",
                re.VERBOSE,
            ),
            "setval": "summary-address "
            "{{ summary_address.address + ' ' + summary_address.mask }} not-advertise",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "summary_address": {
                            "address": "{{ address }}",
                            "mask": "{{ mask }}",
                            "not_advertise": True,
                        },
                    },
                },
            },
        },
        {
            "name": "summary_address.nssa_only",
            "getval": re.compile(
                r"""
                \ssummary-address
                (\s(?P<address>\S+))
                (\s(?P<mask>\S+))
                (\s(?P<nssa>nssa-only))
                (\stag\s(?P<tag>\d+))?
                $""",
                re.VERBOSE,
            ),
            "setval": "summary-address "
            "{{ summary_address.address + ' ' + summary_address.mask }} nssa-only"
            "{{ ' tag ' + summary_address.tag|string if summary_address.tag is defined else '' }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "summary_address": {
                            "address": "{{ address }}",
                            "mask": "{{ mask }}",
                            "nssa_only": True,
                            "tag": "{{ tag }}",
                        },
                    },
                },
            },
        },
        {
            "name": "timers.lsa",
            "getval": re.compile(
                r"""
                \stimers
                \slsa
                \sarrival
                (\s(?P<lsa>\d+))?
                $""",
                re.VERBOSE,
            ),
            "setval": "timers lsa arrival {{ timers.lsa }}",
            "result": {"processes": {"{{ pid }}": {"timers": {"lsa": "{{ lsa }}"}}}},
        },
        {
            "name": "timers.pacing.flood",
            "getval": re.compile(
                r"""
                \stimers\spacing
                (\sflood\s(?P<flood>\d+))
                $""",
                re.VERBOSE,
            ),
            "setval": "timers pacing"
            "{{ ' flood ' + timers.pacing.flood|string if timers.pacing.flood is defined else '' }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "timers": {
                            "pacing": {
                                "flood": "{{ flood }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "timers.pacing.lsa_group",
            "getval": re.compile(
                r"""
                \stimers\spacing
                (\slsa-group\s(?P<lsa_group>\d+))
                $""",
                re.VERBOSE,
            ),
            "setval": "timers pacing"
            "{{ ' lsa-group ' + timers.pacing.lsa_group|string if timers.pacing.lsa_group is defined else '' }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "timers": {
                            "pacing": {
                                "lsa_group": "{{ lsa_group }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "timers.pacing.retransmission",
            "getval": re.compile(
                r"""
                \stimers\spacing
                (\sretransmission\s(?P<retransmission>\d+))?
                $""",
                re.VERBOSE,
            ),
            "setval": "timers pacing"
            "{{ ' retransmission ' + timers.pacing.retransmission|string if timers.pacing.retransmission is defined else '' }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "timers": {
                            "pacing": {
                                "retransmission": "{{ retransmission }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "timers.throttle.lsa",
            "getval": re.compile(
                r"""
                \stimers\sthrottle
                (\s(?P<lsa>lsa))?
                (\s(?P<first_delay>\d+))?
                (\s(?P<min_delay>\d+))?
                (\s(?P<max_delay>\d+))?
                $""",
                re.VERBOSE,
            ),
            "setval": "timers throttle lsa {{ throttle.lsa.first_delay }} {{ throttle.lsa.min_delay }} {{ throttle.lsa.max_delay }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "timers": {
                            "throttle": {
                                "lsa": {
                                    "first_delay": "{{ first_delay }}",
                                    "min_delay": "{{ min_delay }}",
                                    "max_delay": "{{ max_delay }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "timers.throttle.spf",
            "getval": re.compile(
                r"""
                \stimers\sthrottle
                (\s(?P<spf>spf))?
                (\s(?P<first_delay>\d+))?
                (\s(?P<min_delay>\d+))?
                (\s(?P<max_delay>\d+))?
                $""",
                re.VERBOSE,
            ),
            "setval": "timers throttle spf {{ throttle.spf.receive_delay }} {{ throttle.spf.between_delay }} {{ throttle.spf.max_delay }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "timers": {
                            "throttle": {
                                "spf": {
                                    "receive_delay": "{{ first_delay }}",
                                    "between_delay": "{{ min_delay }}",
                                    "max_delay": "{{ max_delay }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "traffic_share",
            "getval": re.compile(
                r"""
                \s(?P<traffic>traffic-share\smin\sacross-interfaces)
                $""",
                re.VERBOSE,
            ),
            "setval": "traffic-share min across-interfaces",
            "result": {
                "processes": {"{{ pid }}": {"traffic_share": "{{ True if traffic is defined }}"}},
            },
        },
        {
            "name": "ttl_security",
            "getval": re.compile(
                r"""
                \sttl-security
                (\s(?P<interfaces>all-interfaces))?
                (\shops\s(?P<hops>\d+))?
                $""",
                re.VERBOSE,
            ),
            "setval": "ttl-security all-interfaces"
            "{{ ' hops ' + ttl_security.hops|string if ttl_security.hops is defined }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "ttl_security": {
                            "set": "{{ True if interfaces is defined and hops is undefined }}",
                            "hops": "{{ hops }}",
                        },
                    },
                },
            },
        },
    ]



# Copyright (c) 2015, Ensighten <infra@ensighten.com>
# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import annotations

DOCUMENTATION = r"""
author: Unknown (!UNKNOWN)
name: credstash
short_description: Retrieve secrets from Credstash on AWS
requirements:
  - credstash (python library)
description:
  - "Credstash is a small utility for managing secrets using AWS's KMS and DynamoDB: https://github.com/fugue/credstash."
options:
  _terms:
    description: Term or list of terms to lookup in the credit store.
    type: list
    elements: string
    required: true
  table:
    description: Name of the credstash table to query.
    type: str
    default: 'credential-store'
  version:
    description: Credstash version.
    type: str
    default: ''
  region:
    description: AWS region.
    type: str
  profile_name:
    description: AWS profile to use for authentication.
    type: str
    env:
      - name: AWS_PROFILE
  aws_access_key_id:
    description: AWS access key ID.
    type: str
    env:
      - name: AWS_ACCESS_KEY_ID
  aws_secret_access_key:
    description: AWS access key.
    type: str
    env:
      - name: AWS_SECRET_ACCESS_KEY
  aws_session_token:
    description: AWS session token.
    type: str
    env:
      - name: AWS_SESSION_TOKEN
"""

EXAMPLES = r"""
- name: first use credstash to store your secrets
  ansible.builtin.shell: credstash put my-github-password secure123

- name: "Test credstash lookup plugin -- get my github password"
  ansible.builtin.debug:
    msg: "Credstash lookup! {{ lookup('community.general.credstash', 'my-github-password') }}"

- name: "Test credstash lookup plugin -- get my other password from us-west-1"
  ansible.builtin.debug:
    msg: "Credstash lookup! {{ lookup('community.general.credstash', 'my-other-password', region='us-west-1') }}"

- name: "Test credstash lookup plugin -- get the company's github password"
  ansible.builtin.debug:
    msg: "Credstash lookup! {{ lookup('community.general.credstash', 'company-github-password', table='company-passwords') }}"

- name: Example play using the 'context' feature
  hosts: localhost
  vars:
    context:
      app: my_app
      environment: production
  tasks:

    - name: "Test credstash lookup plugin -- get the password with a context passed as a variable"
      ansible.builtin.debug:
        msg: "{{ lookup('community.general.credstash', 'some-password', context=context) }}"

    - name: "Test credstash lookup plugin -- get the password with a context defined here"
      ansible.builtin.debug:
        msg: "{{ lookup('community.general.credstash', 'some-password', context=dict(app='my_app', environment='production')) }}"
"""

RETURN = r"""
_raw:
  description:
    - Value(s) stored in Credstash.
  type: str
"""

from ansible.errors import AnsibleError
from ansible.plugins.lookup import LookupBase

from ansible_collections.community.general.plugins.plugin_utils._lookup import check_for_wrong_terms

CREDSTASH_INSTALLED = False

try:
    import credstash

    CREDSTASH_INSTALLED = True
except ImportError:
    CREDSTASH_INSTALLED = False


class LookupModule(LookupBase):
    def run(self, terms, variables=None, **kwargs):
        if not CREDSTASH_INSTALLED:
            raise AnsibleError("The credstash lookup plugin requires credstash to be installed.")

        self.set_options(var_options=variables, direct=kwargs)
        check_for_wrong_terms(self, direct=kwargs)

        version = self.get_option("version")
        region = self.get_option("region")
        table = self.get_option("table")
        profile_name = self.get_option("profile_name")
        aws_access_key_id = self.get_option("aws_access_key_id")
        aws_secret_access_key = self.get_option("aws_secret_access_key")
        aws_session_token = self.get_option("aws_session_token")

        context = {
            k: v
            for k, v in kwargs.items()
            if k
            not in (
                "version",
                "region",
                "table",
                "profile_name",
                "aws_access_key_id",
                "aws_secret_access_key",
                "aws_session_token",
            )
        }

        kwargs_pass = {
            "profile_name": profile_name,
            "aws_access_key_id": aws_access_key_id,
            "aws_secret_access_key": aws_secret_access_key,
            "aws_session_token": aws_session_token,
        }

        ret = []
        for term in terms:
            try:
                ret.append(credstash.getSecret(term, version, region, table, context=context, **kwargs_pass))
            except credstash.ItemNotFound as e:
                raise AnsibleError(f"Key {term} not found") from e
            except Exception as e:
                raise AnsibleError(f"Encountered exception while fetching {term}: {e}") from e

        return ret



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The module file for iosxr_facts
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: iosxr_facts
short_description: Module to collect facts from remote devices.
extends_documentation_fragment:
- cisco.iosxr.iosxr
description:
- Collects facts from network devices running the iosxr operating system. This module
  places the facts gathered in the fact tree keyed by the respective resource name.  The
  facts module will always collect a base set of facts from the device and can enable
  or disable collection of additional facts.
version_added: 1.0.0
notes:
- This module works with connection C(network_cli). See L(the IOS-XR Platform Options,../network/user_guide/platform_iosxr.html).
author:
- Ricardo Carrillo Cruz (@rcarrillocruz)
- Nilashish Chakraborty (@Nilashishc)
options:
  gather_subset:
    description:
    - When supplied, this argument will restrict the facts collected to a given subset.  Possible
      values for this argument include all, hardware, config, and interfaces.  Can
      specify a list of values to include a larger subset.  Values can also be used
      with an initial C(!) to specify that a specific subset should not be collected.
    required: false
    default: 'min'
    type: list
    elements: str
  gather_network_resources:
    description:
    - When supplied, this argument will restrict the facts collected to a given subset.
      Possible values for this argument include all and the resources like interfaces,
      lacp etc. Can specify a list of values to include a larger subset. Values can
      also be used with an initial C(!) to specify that a specific subset should
      not be collected. Valid subsets are 'all', 'lacp', 'lacp_interfaces', 'lldp_global',
      'lldp_interfaces', 'interfaces', 'l2_interfaces', 'l3_interfaces', 'lag_interfaces',
      'acls', 'acl_interfaces', 'static_routes', 'ospfv2'.
    required: false
    type: list
    elements: str
  available_network_resources:
    description: When 'True' a list of network resources for which resource modules are available will be provided.
    type: bool
    default: false
"""

EXAMPLES = """
# Gather all facts
- cisco.iosxr.iosxr_facts:
    gather_subset: all
    gather_network_resources: all

# Collect only the config and default facts
- cisco.iosxr.iosxr_facts:
    gather_subset:
      - config

# Do not collect hardware facts
- cisco.iosxr.iosxr_facts:
    gather_subset:
      - '!hardware'

# Collect only the lacp facts
- cisco.iosxr.iosxr_facts:
    gather_subset:
      - '!all'
      - '!min'
    gather_network_resources:
      - lacp

# Do not collect lacp_interfaces facts
- cisco.iosxr.iosxr_facts:
    gather_network_resources:
      - '!lacp_interfaces'

# Collect lacp and minimal default facts
- cisco.iosxr.iosxr_facts:
    gather_subset: min
    gather_network_resources: lacp

# Collect only the interfaces facts
- cisco.iosxr.iosxr_facts:
    gather_subset:
      - '!all'
      - '!min'
    gather_network_resources:
      - interfaces
      - l2_interfaces
"""

RETURN = """
ansible_net_gather_subset:
  description: The list of fact subsets collected from the device
  returned: always
  type: list

# default
ansible_net_version:
  description: The operating system version running on the remote device
  returned: always
  type: str
ansible_net_hostname:
  description: The configured hostname of the device
  returned: always
  type: str
ansible_net_image:
  description: The image file the device is running
  returned: always
  type: str
ansible_net_api:
  description: The name of the transport
  returned: always
  type: str
ansible_net_python_version:
  description: The Python version Ansible controller is using
  returned: always
  type: str
ansible_net_model:
  description: The model name returned from the device
  returned: always
  type: str

# hardware
ansible_net_filesystems:
  description: All file system names available on the device
  returned: when hardware is configured
  type: list
ansible_net_memfree_mb:
  description: The available free memory on the remote device in Mb
  returned: when hardware is configured
  type: int
ansible_net_memtotal_mb:
  description: The total memory on the remote device in Mb
  returned: when hardware is configured
  type: int
ansible_net_cpu_utilization:
  description: The current CPU utilization of the device
  returned: when hardware is configured
  type: dict

# config
ansible_net_config:
  description: The current active config from the device
  returned: when config is configured
  type: str

# interfaces
ansible_net_all_ipv4_addresses:
  description: All IPv4 addresses configured on the device
  returned: when interfaces is configured
  type: list
ansible_net_all_ipv6_addresses:
  description: All IPv6 addresses configured on the device
  returned: when interfaces is configured
  type: list
ansible_net_interfaces:
  description: A hash of all interfaces running on the system
  returned: when interfaces is configured
  type: dict
ansible_net_neighbors:
  description: The list of LLDP and CDP neighbors from the remote device. If both,
      CDP and LLDP neighbor data is present on one port, CDP is preferred.
  returned: when interfaces is configured
  type: dict

# network resources
ansible_net_gather_network_resources:
  description: The list of fact resource subsets collected from the device
  returned: always
  type: list
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.argspec.facts.facts import (
    FactsArgs,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.facts.facts import (
    FACT_RESOURCE_SUBSETS,
    Facts,
)


def main():
    """
    Main entry point for module execution

    :returns: ansible_facts
    """
    argument_spec = FactsArgs.argument_spec

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    warnings = []

    ansible_facts = {}
    if module.params.get("available_network_resources"):
        ansible_facts["available_network_resources"] = sorted(FACT_RESOURCE_SUBSETS.keys())
    result = Facts(module).get_facts()
    additional_facts, additional_warnings = result
    ansible_facts.update(additional_facts)
    warnings.extend(additional_warnings)

    module.exit_json(ansible_facts=ansible_facts, warnings=warnings)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2023, Gaspard Micol (@gmicol) <gmicol@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: aci_l3out_dhcp_relay_label
short_description: Manage Layer 3 Outside (L3Out) DHCP Relay Label (dhcp:Lbl)
description:
- Manage DHCP Relay Labels for L3Out Logical Interface Profiles on Cisco ACI fabrics.
- A DHCP Relay Label contains the name of an existing DHCP Relay Policy for the label,
  the scope, and a DHCP Option Policy.
options:
  tenant:
    description:
    - The name of an existing tenant.
    type: str
    aliases: [ tenant_name ]
  l3out:
    description:
    - The name of an existing L3Out.
    type: str
    aliases: [ l3out_name ]
  node_profile:
    description:
    - The name of an existing node profile.
    type: str
    aliases: [ node_profile_name, logical_node ]
  interface_profile:
    description:
    - The name of an existing interface profile.
    type: str
    aliases: [ interface_profile_name, logical_interface ]
  dhcp_relay_label:
    description:
    - The name/label of an existing DHCP Relay Policy.
    type: str
    aliases: [ name, relay_policy ]
  scope:
    description:
    - The scope is the owner of the relay server.
    - The APIC defaults to C(infra) when unset during creation.
    type: str
    choices: [ infra, tenant ]
    aliases: [ owner ]
  dhcp_option_policy:
    description:
    - The name of an existing DHCP Option Policy to be associated with the DCHP Relay Policy.
    - The DHCP option policy supplies DHCP clients with configuration parameters
      such as domain, nameserver, and subnet router addresses.
    - Passing an empty string will delete the current linked DHCP Option Policy.
      However, this will associate the DHCP Relay Label to the default DHCP Option Policy
      from the common Tenant.
    type: str
    aliases: [ dhcp_option_policy_name ]
  description:
    description:
    - The description of the DHCP Relay Label.
    type: str
    aliases: [ descr ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

notes:
- The C(tenant), C(l3out), C(node_profile), C(interface_profile) and C(relay_policy) must exist before using this module in your playbook.
  The M(cisco.aci.aci_tenant), M(cisco.aci.aci_l3out), M(cisco.aci.aci_l3out_logical_node_profile), M(cisco.aci.aci_l3out_logical_interface_profile)
  and M(cisco.aci.aci_dhcp_relay) can be used for this.
- If C(dhcp_option_policy) is used, it must exist before using this module in your playbook.
  The M(cisco.aci.aci_dhcp_option_policy) can be used for this.
seealso:
- module: cisco.aci.aci_tenant
- module: cisco.aci.aci_l3out
- module: cisco.aci.aci_l3out_logical_node_profile
- module: cisco.aci.aci_l3out_logical_interface_profile
- module: cisco.aci.aci_dhcp_relay
- module: cisco.aci.aci_dhcp_option_policy
- name: APIC Management Information Model reference
  description: More information about the internal APIC classes
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Gaspard Micol (@gmicol)
"""

EXAMPLES = r"""
- name: Add a new L3Out DHCP Relay Label
  cisco.aci.aci_l3out_eigrp_interface_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    l3out: my_l3out
    node_profile: my_node_profile
    interface_profile: my_interface_profile
    dhcp_relay_label: my_dhcp_relay_label
    scope: tenant
    dhcp_option_policy: my_dhcp_option_policy
    state: present
  delegate_to: localhost

- name: Delete an L3Out DHCP Relay Label
  cisco.aci.aci_l3out_eigrp_interface_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    l3out: my_l3out
    node_profile: my_node_profile
    interface_profile: my_interface_profile
    dhcp_relay_label: my_dhcp_relay_label
    state: absent
  delegate_to: localhost

- name: Query an L3Out DHCP Relay Label
  cisco.aci.aci_l3out_eigrp_interface_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    l3out: my_l3out
    node_profile: my_node_profile
    interface_profile: my_interface_profile
    dhcp_relay_label: my_dhcp_relay_label
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        tenant=dict(type="str", aliases=["tenant_name"]),
        l3out=dict(type="str", aliases=["l3out_name"]),
        node_profile=dict(type="str", aliases=["node_profile_name", "logical_node"]),
        interface_profile=dict(type="str", aliases=["interface_profile_name", "logical_interface"]),
        dhcp_relay_label=dict(type="str", aliases=["name", "relay_policy"]),
        scope=dict(type="str", choices=["infra", "tenant"], aliases=["owner"]),
        dhcp_option_policy=dict(type="str", aliases=["dhcp_option_policy_name"]),
        description=dict(type="str", aliases=["descr"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["tenant", "l3out", "node_profile", "interface_profile", "dhcp_relay_label"]],
            ["state", "present", ["tenant", "l3out", "node_profile", "interface_profile", "dhcp_relay_label"]],
        ],
    )

    tenant = module.params.get("tenant")
    l3out = module.params.get("l3out")
    node_profile = module.params.get("node_profile")
    interface_profile = module.params.get("interface_profile")
    dhcp_relay_label = module.params.get("dhcp_relay_label")
    scope = module.params.get("scope")
    dhcp_option_policy = module.params.get("dhcp_option_policy")
    description = module.params.get("description")
    state = module.params.get("state")

    aci = ACIModule(module)

    child_classes = ["dhcpRsDhcpOptionPol"]

    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="l3extOut",
            aci_rn="out-{0}".format(l3out),
            module_object=l3out,
            target_filter={"name": l3out},
        ),
        subclass_2=dict(
            aci_class="l3extLNodeP",
            aci_rn="lnodep-{0}".format(node_profile),
            module_object=node_profile,
            target_filter={"name": node_profile},
        ),
        subclass_3=dict(
            aci_class="l3extLIfP",
            aci_rn="lifp-[{0}]".format(interface_profile),
            module_object=interface_profile,
            target_filter={"name": interface_profile},
        ),
        subclass_4=dict(
            aci_class="dhcpLbl",
            aci_rn="dhcplbl-[{0}]".format(dhcp_relay_label),
            module_object=dhcp_relay_label,
            target_filter={"name": dhcp_relay_label},
        ),
        child_classes=child_classes,
    )

    aci.get_existing()

    if state == "present":
        child_configs = [dict(dhcpRsDhcpOptionPol=dict(attributes=dict(tnDhcpOptionPolName=dhcp_option_policy)))]

        aci.payload(
            aci_class="dhcpLbl",
            class_config=dict(
                descr=description,
                name=dhcp_relay_label,
                owner=scope,
            ),
            child_configs=child_configs,
        )

        aci.get_diff(aci_class="dhcpLbl")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



# Copyright (c) 2026, Dexter Le <dextersydney2001@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this doc fragment is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations


class ModuleDocFragment:
    # Common parameters for Kopia modules
    DOCUMENTATION = r"""
options:
  password:
    description:
      - Repository password used to encrypt and decrypt repository contents.
    type: str
  config:
    description:
      - Path to the Kopia config file for this repository connection.
      - Defaults to the Kopia default config path when not set.
    type: path
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_policies_global_group_policies_appliance_vlans_assignments_by_vlan_info
short_description: Information module for organizations _policies _global _group _policies
  _appliance _vlans _assignments _by _vlan
description:
  - Get all organizations _policies _global _group _policies _appliance _vlans _assignments
    _by _vlan.
  - List policies by appliance VLANs.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  search:
    description:
      - Search query parameter. Search term for filtering policies.
    type: str
  vlanIds:
    description:
      - VlanIds query parameter. Filter by VLAN IDs.
    elements: str
    type: list
  interfaceIds:
    description:
      - InterfaceIds query parameter. Filter by interface IDs.
    elements: str
    type: list
  perPage:
    description:
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 100. Default is 100.
    type: int
  startingAfter:
    description:
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for appliance getOrganizationPoliciesGlobalGroupPoliciesApplianceVlansAssignmentsByVlan
    description: Complete reference of the getOrganizationPoliciesGlobalGroupPoliciesApplianceVlansAssignmentsByVlan
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-policies-global-group-policies-appliance-vlans-assignments-by-vlan
notes:
  - SDK Method used are
    appliance.Appliance.get_organization_policies_global_group_policies_appliance_vlans_assignments_by_vlan,
  - Paths used are
    get /organizations/{organizationId}/policies/global/group/policies/appliance/vlans/assignments/byVlan,
"""

EXAMPLES = r"""
- name: Get all organizations _policies _global _group _policies _appliance _vlans
    _assignments _by _vlan
  cisco.meraki.organizations_policies_global_group_policies_appliance_vlans_assignments_by_vlan_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    search: string
    vlanIds: []
    interfaceIds: []
    perPage: 0
    startingAfter: string
    endingBefore: string
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "items": [
        {
          "network": {
            "id": "string",
            "name": "string"
          },
          "name": "string",
          "subnet": "string",
          "interfaceId": "string",
          "vlanId": "string",
          "policy": {
            "id": "string",
            "name": "string",
            "group": {
              "number": 0
            }
          }
        }
      ],
      "meta": {
        "counts": {
          "items": {
            "total": 0,
            "remaining": 0
          }
        }
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_policy_objects
short_description: Resource module for organizations _policy _objects
description:
  - Manage operations create, update and delete of the resource organizations _policy
    _objects.
  - Creates a new Policy Object.
  - Deletes a Policy Object.
  - Updates a Policy Object.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  category:
    description: Category of a policy object (one of adaptivePolicy, network).
    type: str
  cidr:
    description: CIDR Value of a policy object (e.g. 10.11.12.1/24").
    type: str
  fqdn:
    description: Fully qualified domain name of policy object (e.g. "example.com").
    type: str
  groupIds:
    description: The IDs of policy object groups the policy object belongs to.
    elements: str
    type: list
  ip:
    description: IP Address of a policy object (e.g. "1.2.3.4").
    type: str
  mask:
    description: Mask of a policy object (e.g. "255.255.0.0").
    type: str
  name:
    description: Name of a policy object, unique within the organization (alphanumeric,
      space, dash, or underscore characters only).
    type: str
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
  policyObjectId:
    description: PolicyObjectId path parameter. Policy object ID.
    type: str
  type:
    description: Type of a policy object (one of adaptivePolicyIpv4Cidr, cidr, fqdn,
      ipAndMask).
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations createOrganizationPolicyObject
    description: Complete reference of the createOrganizationPolicyObject API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-organization-policy-object
  - name: Cisco Meraki documentation for organizations deleteOrganizationPolicyObject
    description: Complete reference of the deleteOrganizationPolicyObject API.
    link: https://developer.cisco.com/meraki/api-v1/#!delete-organization-policy-object
  - name: Cisco Meraki documentation for organizations updateOrganizationPolicyObject
    description: Complete reference of the updateOrganizationPolicyObject API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-organization-policy-object
notes:
  - SDK Method used are
    organizations.Organizations.create_organization_policy_object,
    organizations.Organizations.delete_organization_policy_object,
    organizations.Organizations.update_organization_policy_object,
  - Paths used are
    post /organizations/{organizationId}/policyObjects,
    delete /organizations/{organizationId}/policyObjects/{policyObjectId},
    put /organizations/{organizationId}/policyObjects/{policyObjectId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.organizations_policy_objects:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    category: network
    cidr: 10.0.0.0/24
    fqdn: example.com
    groupIds:
      - '8'
    ip: 1.2.3.4
    mask: 255.255.0.0
    name: Web Servers - Datacenter 10
    organizationId: string
    type: cidr
- name: Update by id
  cisco.meraki.organizations_policy_objects:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    cidr: 10.0.0.0/24
    fqdn: example.com
    groupIds:
      - '8'
    ip: 1.2.3.4
    mask: 255.255.0.0
    name: Web Servers - Datacenter 10
    organizationId: string
    policyObjectId: string
- name: Delete by id
  cisco.meraki.organizations_policy_objects:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: absent
    organizationId: string
    policyObjectId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "name": "string",
      "category": "string",
      "type": "string",
      "cidr": "string",
      "createdAt": "string",
      "updatedAt": "string",
      "groupIds": [
        "string"
      ],
      "networkIds": [
        "string"
      ]
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2017, Ansible by Red Hat, inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: iosxr_banner
author:
- Trishna Guha (@trishnaguha)
- Kedar Kekan (@kedarX)
short_description: Module to configure multiline banners.
description:
- This module will configure both exec and motd banners on remote device running Cisco
  IOS XR. It allows playbooks to add or remove banner text from the running configuration.
version_added: 1.0.0
requirements:
- ncclient >= 0.5.3 when using netconf
- lxml >= 4.1.1 when using netconf
extends_documentation_fragment:
- cisco.iosxr.iosxr
notes:
- This module works with connection C(network_cli) and C(netconf). See L(the IOS-XR
  Platform Options,../network/user_guide/platform_iosxr.html).
options:
  banner:
    description:
    - Specifies the type of banner to configure on remote device.
    required: true
    type: str
    choices:
    - login
    - motd
  text:
    description:
    - Banner text to be configured. Accepts multi line string, without empty lines.
      When using a multi line string, the first and last characters must be the
      start and end delimiters for the banner
      Requires I(state=present).
    type: str
  state:
    description:
    - Existential state of the configuration on the device.
    default: present
    type: str
    choices:
    - present
    - absent
"""

EXAMPLES = """
- name: configure the login banner
  cisco.iosxr.iosxr_banner:
    banner: login
    text: |
      @this is my login banner
      that contains a multiline
      string@
    state: present
- name: remove the motd banner
  cisco.iosxr.iosxr_banner:
    banner: motd
    state: absent
- name: Configure banner from file
  cisco.iosxr.iosxr_banner:
    banner: motd
    text: '{{ lookup(''file'', ''./config_partial/raw_banner.cfg'') }}'
    state: present
"""

RETURN = """
commands:
  description: The list of configuration mode commands sent to device with transport C(cli)
  returned: always (empty list when no commands to send)
  type: list
  sample:
    - banner login
    - "@this is my login banner"
    - that contains a multiline
    - string@

xml:
  description: NetConf rpc xml sent to device with transport C(netconf)
  returned: always (empty list when no xml rpc to send)
  type: list
  sample:
    - '<config xmlns:xc="urn:ietf:params:xml:ns:netconf:base:1.0">
            <banners xmlns="http://cisco.com/ns/yang/Cisco-IOS-XR-infra-infra-cfg">
                <banner xc:operation="merge">
                  <banner-name>motd</banner-name>
                  <banner-text>Ansible banner example</banner-text>
                </banner>
            </banners>
        </config>'
"""

import collections
import re

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.iosxr import (
    build_xml,
    etree_find,
    get_config,
    is_cliconf,
    is_netconf,
    load_config,
)


class ConfigBase(object):
    def __init__(self, module):
        self._module = module
        self._result = {"changed": False, "warnings": []}
        self._want = {}
        self._have = {}

    def map_params_to_obj(self):
        text = self._module.params["text"]
        if text:
            text = "{0}".format(str(text).strip())
        self._want.update(
            {
                "banner": self._module.params["banner"],
                "text": text,
                "state": self._module.params["state"],
            },
        )


class CliConfiguration(ConfigBase):
    def __init__(self, module):
        super(CliConfiguration, self).__init__(module)

    def map_obj_to_commands(self):
        commands = list()
        state = self._module.params["state"]
        if state == "absent":
            if self._have.get("state") != "absent" and (
                "text" in self._have.keys() and self._have["text"]
            ):
                commands.append("no banner {0!s}".format(self._module.params["banner"]))
        elif state == "present":
            if self._want["text"] and self._want["text"].encode().decode(
                "unicode_escape",
            ) != self._have.get("text"):
                banner_cmd = "banner {0!s} ".format(self._module.params["banner"])
                banner_cmd += self._want["text"].strip()
                commands.append(banner_cmd)
        self._result["commands"] = commands
        if commands:
            commit = not self._module.check_mode
            diff = load_config(self._module, commands, commit=commit)
            if diff:
                self._result["diff"] = dict(prepared=diff)
            self._result["changed"] = True

    def map_config_to_obj(self):
        cli_filter = "banner {0!s}".format(self._module.params["banner"])
        output = get_config(self._module, config_filter=cli_filter)
        match = re.search(r"banner (\S+) (.*)", output, re.DOTALL)
        if match:
            text = match.group(2)
        else:
            text = None
        obj = {"banner": self._module.params["banner"], "state": "absent"}
        if output:
            obj["text"] = text
            obj["state"] = "present"
        self._have.update(obj)

    def run(self):
        self.map_params_to_obj()
        self.map_config_to_obj()
        self.map_obj_to_commands()

        return self._result


class NCConfiguration(ConfigBase):
    def __init__(self, module):
        super(NCConfiguration, self).__init__(module)
        self._banners_meta = collections.OrderedDict()
        self._banners_meta.update(
            [
                ("banner", {"xpath": "banners/banner", "tag": True, "attrib": "operation"}),
                ("a:banner", {"xpath": "banner/banner-name"}),
                ("a:text", {"xpath": "banner/banner-text", "operation": "edit"}),
            ],
        )

    def map_obj_to_xml_rpc(self):
        state = self._module.params["state"]
        _get_filter = build_xml(
            "banners",
            xmap=self._banners_meta,
            params=self._module.params,
            opcode="filter",
        )

        running = get_config(self._module, source="running", config_filter=_get_filter)

        banner_name = None
        banner_text = None
        if etree_find(running, "banner-text") is not None:
            banner_name = etree_find(running, "banner-name").text
            banner_text = etree_find(running, "banner-text").text

        opcode = None
        if state == "absent" and banner_name == self._module.params["banner"] and len(banner_text):
            opcode = "delete"
        elif state == "present":
            opcode = "merge"

        self._result["xml"] = []
        if opcode:
            _edit_filter = build_xml(
                "banners",
                xmap=self._banners_meta,
                params=self._module.params,
                opcode=opcode,
            )

            if _edit_filter is not None:
                commit = not self._module.check_mode
                diff = load_config(
                    self._module,
                    _edit_filter,
                    commit=commit,
                    running=running,
                    nc_get_filter=_get_filter,
                )

                if diff:
                    self._result["xml"] = _edit_filter
                    if self._module._diff:
                        self._result["diff"] = dict(prepared=diff)

                    self._result["changed"] = True

    def run(self):
        self.map_params_to_obj()
        self.map_obj_to_xml_rpc()

        return self._result


def main():
    """main entry point for module execution"""
    argument_spec = dict(
        banner=dict(required=True, choices=["login", "motd"]),
        text=dict(),
        state=dict(default="present", choices=["present", "absent"]),
    )

    required_if = [("state", "present", ("text",))]

    module = AnsibleModule(
        argument_spec=argument_spec,
        required_if=required_if,
        supports_check_mode=True,
    )

    config_object = None
    if is_cliconf(module):
        # Commenting the below cliconf deprecation support call for Ansible 2.9 as it'll be continued to be supported
        config_object = CliConfiguration(module)
    elif is_netconf(module):
        config_object = NCConfiguration(module)

    result = None
    if config_object is not None:
        result = config_object.run()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from ansible_collections.ansible.mysql.plugins.module_utils.version import LooseVersion
from ansible_collections.ansible.mysql.plugins.module_utils.mysql import get_server_version


def use_old_user_mgmt(cursor):
    version = get_server_version(cursor)

    return LooseVersion(version) < LooseVersion("10.2")


def supports_identified_by_password(cursor):
    return True


def server_supports_alter_user(cursor):
    version = get_server_version(cursor)

    return LooseVersion(version) >= LooseVersion("10.2")


def server_supports_password_expire(cursor):
    version = get_server_version(cursor)

    return LooseVersion(version) >= LooseVersion("10.4.3")


def get_tls_requires(cursor, user, host):
    """Get user TLS requirements.
    Reads directly from mysql.user table allowing for a more
    readable code.

    Args:
        cursor (cursor): DB driver cursor object.
        user (str): User name.
        host (str): User host name.

    Returns: Dictionary containing current TLS required
    """
    tls_requires = dict()

    query = ('SELECT ssl_type, ssl_cipher, x509_issuer, x509_subject '
             'FROM mysql.user WHERE User = %s AND Host = %s')
    cursor.execute(query, (user, host))
    res = cursor.fetchone()

    # Mysql_info use a DictCursor so we must convert back to a list
    # otherwise we get KeyError 0
    if isinstance(res, dict):
        res = list(res.values())

    # When user don't require SSL, res value is: ('', '', '', '')
    if not any(res):
        return None

    if res[0] == 'ANY':
        tls_requires['SSL'] = None

    if res[0] == 'X509':
        tls_requires['X509'] = None

    if res[1]:
        tls_requires['CIPHER'] = res[1]

    if res[2]:
        tls_requires['ISSUER'] = res[2]

    if res[3]:
        tls_requires['SUBJECT'] = res[3]
    return tls_requires



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_lan_connectivity_policy
short_description: Manage LAN Connectivity Policies and vNICs for Cisco Intersight
description:
  - Create, update, and delete LAN Connectivity Policies on Cisco Intersight.
  - Manage individual vNICs within LAN Connectivity policies.
  - Supports both Standalone and FIAttached target platforms with different configuration options.
  - LAN Connectivity policies define network connectivity settings for server profiles.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/vnic/LanConnectivityPolicy/get/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Policies created within a Custom Organization are applicable only to devices in the same Organization.
      - Use 'default' for the default organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the LAN Connectivity Policy.
      - Must be unique within the organization.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the LAN Connectivity Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  target_platform:
    description:
      - The platform type for which the LAN Connectivity policy is intended.
      - Standalone for Standalone servers, FIAttached for fabric interconnect attached servers.
    type: str
    choices: ['Standalone', 'FIAttached']
    default: 'Standalone'
  azure_qos_enabled:
    description:
      - Enable Azure QoS for the LAN Connectivity policy.
      - Only applicable when target_platform is 'FIAttached'.
    type: bool
    default: false
  iqn_allocation_type:
    description:
      - IQN allocation type for the LAN Connectivity policy.
      - Only applicable when target_platform is 'FIAttached'.
    type: str
    choices: ['None', 'Pool', 'Static']
    default: 'None'
  placement_mode:
    description:
      - Placement mode for vNIC assignment.
      - Only applicable when target_platform is 'FIAttached'.
    type: str
    choices: ['custom', 'auto']
    default: 'custom'
  iqn_pool_name:
    description:
      - Relationship to the iSCSI Qualified Name Pool.
      - Required when iqn_allocation_type is 'Pool'.
      - Only applicable for FIAttached target platform.
    type: str
  static_iqn_name:
    description:
      - User provided static iSCSI Qualified Name (IQN) for use as initiator identifiers by iSCSI vNICs in a Fabric Interconnect domain.
      - Required when iqn_allocation_type is 'Static'.
      - Only applicable for FIAttached target platform.
    type: str
  vnics:
    description:
      - List of vNICs to be created and attached to the LAN Connectivity policy.
      - Each vNIC will be configured with the specified network and adapter policies.
      - Required when C(state) is C(present).
      - At least one vNIC must be specified for both Standalone and FIAttached platforms.
    type: list
    elements: dict
    suboptions:
      name:
        description:
          - The name of the vNIC.
          - Must be unique within the LAN Connectivity policy.
        type: str
        required: true
      placement_slot_id:
        description:
          - PCIe Slot where the VIC adapter is installed.
          - Supported values are (1-15) and MLOM.
          - Required when vNIC state is 'present'.
        type: str
      uplink_port:
        description:
          - Adapter port on which the virtual interface will be created.
        type: int
        choices: [0, 1, 2, 3]
        default: 0
      order:
        description:
          - The order in which the virtual interface is brought up.
          - The order assigned to an interface should be unique for all the Ethernet and Fibre-Channel interfaces on each PCI link on a VIC adapter.
          - The order should start from zero with no overlaps.
          - The maximum value of PCI order is limited by the number of virtual interfaces (Ethernet and Fibre-Channel) on each PCI link on a VIC adapter.
          - All VIC adapters have a single PCI link except VIC 1340, VIC 1380 and VIC 1385 which have two.
        type: int
        default: 0
      cdn_source:
        description:
          - Source of the CDN. It can either be user specified or be the same as the vNIC name.
        type: str
        choices: ['vnic', 'user']
        default: 'vnic'
      cdn_value:
        description:
          - CDN value when cdn_source is 'user'.
        type: str
      eth_network_policy_name:
        description:
          - Relationship to the Ethernet Network Policy.
          - Required when vNIC state is 'present'.
        type: str
      eth_qos_policy_name:
        description:
          - Relationship to the Ethernet QoS Policy.
          - Required when vNIC state is 'present'.
        type: str
      eth_adapter_policy_name:
        description:
          - Relationship to the Ethernet Adapter Policy.
          - Required when vNIC state is 'present'.
        type: str
      connection_type:
        description:
          - Type of connection for the vNIC.
        type: str
        choices: ['none', 'usnic', 'vmq', 'sriov']
        default: 'none'
      mac_address_type:
        description:
          - Type of MAC address assignment.
          - Only applicable for FIAttached target platform.
        type: str
        choices: ['pool', 'static']
        default: 'pool'
      mac_pool_name:
        description:
          - The MAC pool that is assigned.
          - Required when mac_address_type is 'pool' and target platform is FIAttached.
          - Only applicable for FIAttached target platform.
        type: str
      static_mac_address:
        description:
          - The MAC address must be in hexadecimal format xx:xx:xx:xx:xx:xx.
          - To ensure uniqueness of MACs in the LAN fabric, you are strongly encouraged to use the following MAC prefix 00:25:B5:xx:xx:xx.
          - Required when mac_address_type is 'static' and target platform is FIAttached.
          - Only applicable for FIAttached target platform.
        type: str
      pci_link:
        description:
          - The PCI Link used as transport for the virtual interface.
          - PCI Link is only applicable for select Cisco UCS VIC 1300 models (UCSC-PCIE-C40Q-03, UCSB-MLOM-40G-03, UCSB-VIC-M83-8P) that support two PCI links.
          - The value, if specified, for any other VIC model will be ignored.
          - For FIAttached required when pci_link_assignment_mode is 'Custom'.
        type: int
        choices: [0, 1]
        default: 0
      auto_slot_id:
        description:
          - Enable or disable automatic assignment of the VIC slot ID.
          - If enabled and the server has only one VIC, the same VIC is chosen for all the vNICs.
          - If enabled and the server has multiple VICs, the vNIC/vHBA are deployed on the first VIC.
          - If disabled, placement_slot_id must be specified.
          - Only applicable for FIAttached target platform when auto_vnic_placement_enabled is false.
        type: bool
        default: true
      auto_pci_link:
        description:
          - Enable or disable automatic assignment of the PCI Link in a dual-link adapter.
          - This option applies only to 13xx series VICs that support dual-link.
          - If enabled, the system determines the placement of the vNIC/vHBA on either of the PCI Links.
          - If disabled, pci_link_assignment_mode must be specified.
          - Only applicable for FIAttached target platform when auto_vnic_placement_enabled is false.
        type: bool
        default: true
      pci_link_assignment_mode:
        description:
          - PCI Link assignment mode when auto_pci_link is disabled.
          - Custom allows manual selection of PCI link via pci_link parameter.
          - Load-Balanced automatically distributes vNICs across available PCI links.
          - Required when auto_pci_link is false and auto_vnic_placement_enabled is false.
          - Only applicable for FIAttached target platform.
        type: str
        choices: ['Custom', 'Load-Balanced']
      auto_vnic_placement_enabled:
        description:
          - Enable automatic vNIC placement for FIAttached servers.
          - When enabled, the vNIC placement is simplified to only specify the switch ID.
          - When disabled, full placement control is available including order, auto_slot_id, and auto_pci_link.
          - Only applicable for FIAttached target platform.
        type: bool
        default: false
      switch_id:
        description:
          - The fabric port to which the vNICs will be associated.
          - Only applicable for FIAttached target platform.
        type: str
        choices: ['A', 'B']
        default: 'A'
      failover_enabled:
        description:
          - Enables automatic vNIC failover to the secondary Fabric Interconnect
            if the primary path fails.
          - Failover applies only to Cisco VICs that are connected to a Fabric Interconnect cluster.
          - Only applicable for FIAttached target platform.
        type: bool
        default: false
      fabric_eth_network_group_policy_name:
        description:
          - Relationship to the Fabric Ethernet Group Policy.
          - Required when target platform is FIAttached and vNIC state is 'present'.
          - Only applicable for FIAttached target platform.
        type: str
      fabric_eth_network_control_policy_name:
        description:
          - Relationship to the Fabric Ethernet Network Policy.
          - Required when target platform is FIAttached and vNIC state is 'present'.
          - Only applicable for FIAttached target platform.
        type: str
      iscsi_boot_policy_name:
        description:
          - Relationship to the boot iSCSI Policy.
          - Only applicable for FIAttached target platform.
        type: str
      pin_group_name:
        description:
          - Pingroup name associated to vNIC for static pinning.
          - LCP deploy will resolve pingroup name and fetches the corresponding uplink port/port channel to pin the vNIC traffic.
          - Only applicable for FIAttached target platform.
        type: str
      usnic_settings:
        description:
          - USNIC settings when connection_type is 'usnic'.
          - Required when connection_type is 'usnic'.
        type: dict
        suboptions:
          count:
            description:
              - Number of usNIC interfaces to be created.
              - When usNIC is enabled, the valid values are from 1 to 225.
              - When usNIC is disabled, the default value is 0.
            type: int
            default: 0
          cos:
            description:
              - Class of Service to be used for traffic on the usNIC.
            type: int
            choices: [0, 1, 2, 3, 4, 5, 6]
            default: 5
          usnic_adapter_policy_name:
            description:
              - Ethernet Adapter policy to be associated with the usNICs.
              - Required when connection_type is 'usnic'.
            type: str
      vmq_settings:
        description:
          - VMQ settings when connection_type is 'vmq'.
        type: dict
        suboptions:
          enabled:
            description:
              - Enable VMQ.
            type: bool
            default: true
          multi_queue_support:
            description:
              - Enables Virtual Machine Multi-Queue feature on the virtual interface.
              - VMMQ allows configuration of multiple I/O queues for a single VM and thus distributes traffic across multiple CPU cores in a VM.
            type: bool
            default: false
          num_interrupts:
            description:
              - The number of interrupt resources to be allocated.
              - Recommended value is the number of CPU threads or logical processors available in the server.
            type: int
            default: 16
          num_vmqs:
            description:
              - The number of hardware Virtual Machine Queues to be allocated.
              - The number of VMQs per adapter must be one more than the maximum number of VM NICs.
            type: int
            default: 4
          num_sub_vnics:
            description:
              - Number of sub vNICs (0-64).
              - Only applicable when multi_queue_support is true.
            type: int
            default: 64
          vmmq_adapter_policy_name:
            description:
              - Name of the VMMQ Adapter Policy.
              - Only applicable when multi_queue_support is true.
            type: str
      sriov_settings:
        description:
          - SR-IOV settings when connection_type is 'sriov'.
        type: dict
        suboptions:
          enabled:
            description:
              - Enable SR-IOV.
            type: bool
            default: true
          vf_count:
            description:
              - Number of Virtual Functions (1-64).
            type: int
            default: 64
          rx_count_per_vf:
            description:
              - Receive Queue Count per VF (1-8).
            type: int
            default: 4
          tx_count_per_vf:
            description:
              - Transmit Queue Count per VF (1-8).
            type: int
            default: 1
          comp_count_per_vf:
            description:
              - Completion Queue Count per VF (1-16).
            type: int
            default: 5
          int_count_per_vf:
            description:
              - Interrupt Count per VF (1-16).
            type: int
            default: 8
      state:
        description:
          - Whether to create/update or delete the vNIC.
        type: str
        choices: ['present', 'absent']
        default: present
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create a LAN Connectivity Policy for Standalone servers
  cisco.intersight.intersight_lan_connectivity_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "Standalone-lan-policy"
    description: "LAN connectivity policy for Standalone servers"
    target_platform: "Standalone"
    tags:
      - Key: "Environment"
        Value: "Production"
      - Key: "Site"
        Value: "DataCenter-A"
    vnics:
      - name: "eth0"
        placement_slot_id: "1"
        pci_link: 0
        uplink_port: 0
        order: 0
        eth_network_policy_name: "default-network-policy"
        eth_qos_policy_name: "default-qos-policy"
        eth_adapter_policy_name: "default-adapter-policy"
        connection_type: "none"
      - name: "eth1"
        placement_slot_id: "2"
        pci_link: 0
        uplink_port: 1
        order: 1
        eth_network_policy_name: "vlan-network-policy"
        eth_qos_policy_name: "high-qos-policy"
        eth_adapter_policy_name: "performance-adapter-policy"
        connection_type: "none"
    state: present

- name: Create a LAN Connectivity Policy for FIAttached servers
  cisco.intersight.intersight_lan_connectivity_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "fi-attached-lan-policy"
    description: "LAN connectivity policy for FIAttached servers"
    target_platform: "FIAttached"
    azure_qos_enabled: true
    iqn_allocation_type: "Pool"
    iqn_pool_name: "default-iqn-pool"
    placement_mode: "custom"
    tags:
      - Key: "Environment"
        Value: "Production"
    vnics:
      - name: "vnic-fi-attached"
        order: 0
        cdn_source: "vnic"
        mac_address_type: "pool"
        mac_pool_name: "default-mac-pool"
        auto_slot_id: true
        auto_pci_link: true
        auto_vnic_placement_enabled: false
        switch_id: "A"
        failover_enabled: false
        fabric_eth_network_group_policy_name: "default-network-group"
        fabric_eth_network_control_policy_name: "default-network-control"
        eth_qos_policy_name: "default-qos-policy"
        eth_adapter_policy_name: "default-adapter-policy"
        connection_type: "none"
        state: present
    state: present

- name: Create a LAN Connectivity Policy with pin group for static pinning
  cisco.intersight.intersight_lan_connectivity_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "fi-attached-pinned-policy"
    description: "LAN connectivity policy with static uplink pinning"
    target_platform: "FIAttached"
    azure_qos_enabled: false
    iqn_allocation_type: "None"
    placement_mode: "custom"
    vnics:
      - name: "pinned-vnic"
        order: 0
        cdn_source: "vnic"
        mac_address_type: "pool"
        mac_pool_name: "default-mac-pool"
        auto_slot_id: true
        auto_pci_link: true
        auto_vnic_placement_enabled: false
        switch_id: "A"
        failover_enabled: false
        fabric_eth_network_group_policy_name: "default-network-group"
        fabric_eth_network_control_policy_name: "default-network-control"
        eth_qos_policy_name: "default-qos-policy"
        eth_adapter_policy_name: "default-adapter-policy"
        pin_group_name: "pingroup-a"
        connection_type: "none"
        state: present
    state: present

- name: Create a LAN Connectivity Policy with automatic vNIC placement
  cisco.intersight.intersight_lan_connectivity_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "fi-attached-auto-placement-policy"
    description: "LAN connectivity policy with automatic vNIC placement"
    target_platform: "FIAttached"
    azure_qos_enabled: false
    iqn_allocation_type: "None"
    placement_mode: "auto"
    vnics:
      - name: "auto-placed-vnic"
        cdn_source: "vnic"
        mac_address_type: "pool"
        mac_pool_name: "default-mac-pool"
        auto_vnic_placement_enabled: true
        switch_id: "A"
        failover_enabled: false
        fabric_eth_network_group_policy_name: "default-network-group"
        fabric_eth_network_control_policy_name: "default-network-control"
        eth_qos_policy_name: "default-qos-policy"
        eth_adapter_policy_name: "default-adapter-policy"
        connection_type: "none"
        state: present
    state: present

- name: Create a LAN Connectivity Policy with advanced placement control
  cisco.intersight.intersight_lan_connectivity_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "fi-attached-advanced-placement-policy"
    description: "LAN connectivity policy with advanced placement control"
    target_platform: "FIAttached"
    azure_qos_enabled: false
    iqn_allocation_type: "None"
    placement_mode: "custom"
    vnics:
      - name: "advanced-placed-vnic"
        order: 9
        cdn_source: "vnic"
        mac_address_type: "pool"
        mac_pool_name: "default-mac-pool"
        auto_slot_id: false
        placement_slot_id: "5"
        auto_pci_link: false
        pci_link_assignment_mode: "Custom"
        pci_link: 0
        auto_vnic_placement_enabled: false
        switch_id: "A"
        failover_enabled: false
        fabric_eth_network_group_policy_name: "default-network-group"
        fabric_eth_network_control_policy_name: "default-network-control"
        eth_qos_policy_name: "default-qos-policy"
        eth_adapter_policy_name: "default-adapter-policy"
        connection_type: "none"
        state: present
    state: present

- name: Create a LAN Connectivity Policy with load-balanced PCI assignment
  cisco.intersight.intersight_lan_connectivity_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "fi-attached-load-balanced-policy"
    description: "LAN connectivity policy with load-balanced PCI assignment"
    target_platform: "FIAttached"
    azure_qos_enabled: false
    iqn_allocation_type: "None"
    placement_mode: "custom"
    vnics:
      - name: "load-balanced-vnic"
        order: 5
        cdn_source: "vnic"
        mac_address_type: "pool"
        mac_pool_name: "default-mac-pool"
        auto_slot_id: false
        placement_slot_id: "MLOM"
        auto_pci_link: false
        pci_link_assignment_mode: "Load-Balanced"
        auto_vnic_placement_enabled: false
        switch_id: "B"
        failover_enabled: true
        fabric_eth_network_group_policy_name: "default-network-group"
        fabric_eth_network_control_policy_name: "default-network-control"
        eth_qos_policy_name: "default-qos-policy"
        eth_adapter_policy_name: "default-adapter-policy"
        connection_type: "none"
        state: present
    state: present

- name: Create a LAN Connectivity Policy with USNIC vNIC
  cisco.intersight.intersight_lan_connectivity_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "usnic-lan-policy"
    description: "Policy with USNIC configuration"
    target_platform: "Standalone"
    vnics:
      - name: "usnic-eth0"
        placement_slot_id: "4"
        pci_link: 0
        uplink_port: 0
        order: 0
        eth_network_policy_name: "hpc-network-policy"
        eth_qos_policy_name: "hpc-qos-policy"
        eth_adapter_policy_name: "hpc-adapter-policy"
        connection_type: "usnic"
        usnic_settings:
          count: 0
          cos: 5
          usnic_adapter_policy_name: "hpc-adapter-policy"
    state: present

- name: Create a LAN Connectivity Policy with VMQ vNIC
  cisco.intersight.intersight_lan_connectivity_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "vmq-lan-policy"
    description: "Policy with VMQ configuration"
    target_platform: "Standalone"
    vnics:
      - name: "vmq-eth0"
        placement_slot_id: "10"
        pci_link: 0
        uplink_port: 2
        order: 0
        eth_network_policy_name: "vm-network-policy"
        eth_qos_policy_name: "vm-qos-policy"
        eth_adapter_policy_name: "vm-adapter-policy"
        connection_type: "vmq"
        vmq_settings:
          enabled: true
          multi_queue_support: false
          num_interrupts: 16
          num_vmqs: 4
    state: present

- name: Create a LAN Connectivity Policy with VMQ multi-queue support
  cisco.intersight.intersight_lan_connectivity_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "vmq-multiqueue-lan-policy"
    description: "Policy with VMQ multi-queue configuration"
    target_platform: "Standalone"
    vnics:
      - name: "vmq-mq-eth0"
        placement_slot_id: "11"
        pci_link: 1
        uplink_port: 3
        order: 0
        eth_network_policy_name: "vm-network-policy"
        eth_qos_policy_name: "vm-qos-policy"
        eth_adapter_policy_name: "vm-adapter-policy"
        connection_type: "vmq"
        vmq_settings:
          enabled: true
          multi_queue_support: true
          num_sub_vnics: 64
          vmmq_adapter_policy_name: "vmmq-adapter-policy"
    state: present

- name: Create a LAN Connectivity Policy with SR-IOV vNIC
  cisco.intersight.intersight_lan_connectivity_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "sriov-lan-policy"
    description: "Policy with SR-IOV configuration"
    target_platform: "Standalone"
    vnics:
      - name: "sriov-eth0"
        placement_slot_id: "14"
        pci_link: 0
        uplink_port: 1
        order: 0
        eth_network_policy_name: "sriov-network-policy"
        eth_qos_policy_name: "sriov-qos-policy"
        eth_adapter_policy_name: "sriov-adapter-policy"
        connection_type: "sriov"
        sriov_settings:
          enabled: true
          vf_count: 64
          rx_count_per_vf: 4
          tx_count_per_vf: 1
          comp_count_per_vf: 5
          int_count_per_vf: 8
    state: present

- name: Create a LAN Connectivity Policy with custom CDN values
  cisco.intersight.intersight_lan_connectivity_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "custom-cdn-lan-policy"
    description: "Policy with custom CDN values"
    target_platform: "Standalone"
    vnics:
      - name: "mgmt-nic"
        placement_slot_id: "1"
        pci_link: 0
        uplink_port: 0
        order: 0
        cdn_source: "user"
        cdn_value: "Management-NIC"
        eth_network_policy_name: "mgmt-network-policy"
        eth_qos_policy_name: "mgmt-qos-policy"
        eth_adapter_policy_name: "mgmt-adapter-policy"
        connection_type: "none"
    state: present

- name: Update LAN connectivity policy - manage vNIC states
  cisco.intersight.intersight_lan_connectivity_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "mixed-vnic-states-policy"
    description: "Policy demonstrating vNIC state management"
    target_platform: "Standalone"
    vnics:
      - name: "eth0"
        placement_slot_id: "1"
        pci_link: 0
        uplink_port: 0
        order: 0
        eth_network_policy_name: "production-network"
        eth_qos_policy_name: "standard-qos"
        eth_adapter_policy_name: "standard-adapter"
        connection_type: "none"
        state: present
      - name: "eth1-old"
        state: absent
      - name: "eth2-new"
        placement_slot_id: "3"
        pci_link: 0
        uplink_port: 1
        order: 1
        eth_network_policy_name: "production-network"
        eth_qos_policy_name: "standard-qos"
        eth_adapter_policy_name: "standard-adapter"
        connection_type: "vmq"
        vmq_settings:
          enabled: true
          multi_queue_support: false
          num_interrupts: 16
          num_vmqs: 4
        state: present
    state: present

- name: Delete a LAN Connectivity Policy
  cisco.intersight.intersight_lan_connectivity_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "old-lan-policy"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "test-lan-policy",
        "ObjectType": "vnic.LanConnectivityPolicy",
        "TargetPlatform": "Standalone",
        "Tags": [
            {
                "Key": "Environment",
                "Value": "Production"
            }
        ],
        "vNICs": [
            {
                "Name": "eth0",
                "ObjectType": "vnic.EthIf",
                "Order": 0,
                "Placement": {
                    "Id": "1"
                }
            }
        ]
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec
from ansible_collections.cisco.intersight.plugins.module_utils.intersight_vnic_utils import (
    validate_cdn_config, validate_usnic_settings, validate_vmq_settings, validate_sriov_settings,
    build_cdn_config, build_connection_settings, resolve_policy_moids_from_mappings,
    get_common_policy_mappings, get_iscsi_boot_policy_mapping, get_fabric_network_policy_mappings,
    get_mac_pool_policy_mapping, get_common_settings_argument_spec
)


def validate_fi_attached_params(module):
    """
    Validate FIAttached specific parameters
    """
    iqn_allocation_type = module.params.get('iqn_allocation_type')
    static_iqn_name = module.params.get('static_iqn_name')
    iqn_pool_name = module.params.get('iqn_pool_name')

    # Validate IQN allocation type requirements
    if iqn_allocation_type == 'Static' and not static_iqn_name:
        module.fail_json(msg="static_iqn_name is required when iqn_allocation_type is 'Static'")

    if iqn_allocation_type == 'Pool' and not iqn_pool_name:
        module.fail_json(msg="iqn_pool_name is required when iqn_allocation_type is 'Pool'")

    if iqn_allocation_type != 'Static' and static_iqn_name:
        module.fail_json(msg="static_iqn_name should only be specified when iqn_allocation_type is 'Static'")

    if iqn_allocation_type != 'Pool' and iqn_pool_name:
        module.fail_json(msg="iqn_pool_name should only be specified when iqn_allocation_type is 'Pool'")


def validate_fi_attached_vnic_config(module, vnic_config):
    """
    Validate FIAttached specific vNIC configuration
    """
    vnic_name = vnic_config.get('name', 'unknown')
    mac_address_type = vnic_config.get('mac_address_type', 'pool')
    mac_pool_name = vnic_config.get('mac_pool_name')
    static_mac_address = vnic_config.get('static_mac_address')

    # Validate MAC address configuration
    if mac_address_type == 'pool' and not mac_pool_name:
        module.fail_json(msg=f"mac_pool_name is required when mac_address_type is 'pool' for vNIC '{vnic_name}'")

    if mac_address_type == 'static' and not static_mac_address:
        module.fail_json(msg=f"static_mac_address is required when mac_address_type is 'static' for vNIC '{vnic_name}'")

    if mac_address_type != 'pool' and mac_pool_name:
        module.fail_json(msg=f"mac_pool_name should only be specified when mac_address_type is 'pool' for vNIC '{vnic_name}'")

    if mac_address_type != 'static' and static_mac_address:
        module.fail_json(msg=f"static_mac_address should only be specified when mac_address_type is 'static' for vNIC '{vnic_name}'")

    # Validate required FIAttached fields
    fabric_eth_network_group_policy_name = vnic_config.get('fabric_eth_network_group_policy_name')
    fabric_eth_network_control_policy_name = vnic_config.get('fabric_eth_network_control_policy_name')

    if not fabric_eth_network_group_policy_name:
        module.fail_json(msg=f"fabric_eth_network_group_policy_name is required for attached vNIC '{vnic_name}'")

    if not fabric_eth_network_control_policy_name:
        module.fail_json(msg=f"fabric_eth_network_control_policy_name is required for attached vNIC '{vnic_name}'")

    # Validate advanced placement configuration
    validate_fi_attached_placement_config(module, vnic_config)


def validate_fi_attached_placement_config(module, vnic_config):
    """
    Validate FIAttached placement configuration
    """
    vnic_name = vnic_config.get('name', 'unknown')
    auto_vnic_placement_enabled = vnic_config.get('auto_vnic_placement_enabled', False)

    # Skip advanced placement validation if auto placement is enabled
    if auto_vnic_placement_enabled:
        return

    auto_slot_id = vnic_config.get('auto_slot_id', True)
    placement_slot_id = vnic_config.get('placement_slot_id')
    auto_pci_link = vnic_config.get('auto_pci_link', True)
    pci_link_assignment_mode = vnic_config.get('pci_link_assignment_mode')
    pci_link = vnic_config.get('pci_link')

    # Validate slot ID configuration
    if not auto_slot_id and not placement_slot_id:
        module.fail_json(msg=f"placement_slot_id is required when auto_slot_id is false for vNIC '{vnic_name}'")

    if auto_slot_id and placement_slot_id:
        module.fail_json(msg=f"placement_slot_id should not be specified when auto_slot_id is true for vNIC '{vnic_name}'")

    # Validate PCI link configuration
    if not auto_pci_link and not pci_link_assignment_mode:
        module.fail_json(msg=f"pci_link_assignment_mode is required when auto_pci_link is false for vNIC '{vnic_name}'")

    if auto_pci_link and pci_link_assignment_mode:
        module.fail_json(msg=f"pci_link_assignment_mode should not be specified when auto_pci_link is true for vNIC '{vnic_name}'")

    # Validate PCI link when using Custom mode
    if pci_link_assignment_mode == 'Custom' and pci_link is None:
        module.fail_json(msg=f"pci_link is required when pci_link_assignment_mode is 'Custom' for vNIC '{vnic_name}'")

    if pci_link_assignment_mode != 'Custom' and pci_link is not None and pci_link != 0:
        module.fail_json(msg=f"pci_link should only be specified when pci_link_assignment_mode is 'Custom' for vNIC '{vnic_name}'")


def validate_standalone_vnic_config(module, vnic_config):
    """
    Validate Standalone specific vNIC configuration
    """
    vnic_name = vnic_config.get('name', 'unknown')

    # Validate required Standalone fields
    placement_slot_id = vnic_config.get('placement_slot_id')
    eth_network_policy_name = vnic_config.get('eth_network_policy_name')

    if not placement_slot_id:
        module.fail_json(msg=f"placement_slot_id is required for Standalone vNIC '{vnic_name}'")

    if not eth_network_policy_name:
        module.fail_json(msg=f"eth_network_policy_name is required for Standalone vNIC '{vnic_name}'")


def get_vnic_policy_mappings(target_platform, vnic_config):
    """
    Get policy mappings for vNIC configuration based on target platform.
    """
    # Start with common policies
    policy_mappings = get_common_policy_mappings()

    # Platform-specific policies
    if target_platform == 'FIAttached':
        # Add fabric network policies
        policy_mappings.update(get_fabric_network_policy_mappings())

        # MAC pool for FIAttached when using pool type
        if vnic_config.get('mac_address_type', 'pool') == 'pool':
            policy_mappings.update(get_mac_pool_policy_mapping())

        # iSCSI boot policy
        if vnic_config.get('iscsi_boot_policy_name'):
            policy_mappings.update(get_iscsi_boot_policy_mapping())

    else:  # Standalone server
        policy_mappings['eth_network_policy_name'] = ('/vnic/EthNetworkPolicies', 'EthNetworkPolicy', 'Ethernet Network Policy')

    return policy_mappings


def resolve_vnic_policy_moids(intersight, policy_cache, module, vnic_config, target_platform):
    """
    Resolve all policy MOIDs for a vNIC configuration based on target platform.
    """
    policy_mappings = get_vnic_policy_mappings(target_platform, vnic_config)
    organization_name = module.params['organization']
    return resolve_policy_moids_from_mappings(intersight, policy_cache, module, vnic_config, policy_mappings, organization_name)


def build_vnic_api_body(intersight, policy_cache, module, vnic_config, lan_connectivity_policy_moid):
    """
    Build vNIC API body for API call
    """
    target_platform = module.params.get('target_platform')

    if target_platform == 'FIAttached':
        return build_fi_attached_vnic_api_body(intersight, policy_cache, module, vnic_config, lan_connectivity_policy_moid)
    else:
        return build_standalone_vnic_api_body(intersight, policy_cache, module, vnic_config, lan_connectivity_policy_moid)


def build_standalone_vnic_api_body(intersight, policy_cache, module, vnic_config, lan_connectivity_policy_moid):
    """
    Build Standalone vNIC API body for Intersight API call
    """
    # Base vNIC configuration for Standalone
    vnic_api_body = {
        'Name': vnic_config['name'],
        'Placement': {
            'Id': vnic_config['placement_slot_id'],
            'Uplink': vnic_config.get('uplink_port', 0),
            'PciLink': vnic_config.get('pci_link', 0)
        },
        'Order': vnic_config.get('order', 0),
        'LanConnectivityPolicy': lan_connectivity_policy_moid
    }

    # Add common CDN configuration
    vnic_api_body['Cdn'] = build_cdn_config(vnic_config)

    # Resolve and add policy MOIDs
    policy_moids = resolve_vnic_policy_moids(intersight, policy_cache, module, vnic_config, 'Standalone')
    vnic_api_body.update(policy_moids)

    # Add common connection type settings
    organization_name = module.params['organization']
    connection_settings = build_connection_settings(intersight, policy_cache, module, vnic_config, organization_name)
    vnic_api_body.update(connection_settings)

    return vnic_api_body


def build_fi_attached_vnic_api_body(intersight, policy_cache, module, vnic_config, lan_connectivity_policy_moid):
    """
    Build FIAttached vNIC API body for Intersight API call
    """
    # Base vNIC configuration for FIAttached
    # Map lowercase user input to API format
    mac_address_type = vnic_config.get('mac_address_type', 'pool')
    api_mac_address_type = 'STATIC' if mac_address_type == 'static' else 'POOL'
    auto_vnic_placement_enabled = vnic_config.get('auto_vnic_placement_enabled', False)

    vnic_api_body = {
        'Name': vnic_config['name'],
        'MacAddressType': api_mac_address_type,
        'FailoverEnabled': vnic_config.get('failover_enabled', False),
        'LanConnectivityPolicy': lan_connectivity_policy_moid
    }

    # Handle placement based on auto_vnic_placement_enabled
    if auto_vnic_placement_enabled:
        vnic_api_body['Placement'] = {
            'SwitchId': vnic_config.get('switch_id', 'A'),
            'AutoSlotId': True,
            'AutoPciLink': True
        }
    else:
        # Full placement control
        placement = {
            'SwitchId': vnic_config.get('switch_id', 'A'),
            'AutoSlotId': vnic_config.get('auto_slot_id', True),
            'AutoPciLink': vnic_config.get('auto_pci_link', True)
        }

        # Add slot ID if auto_slot_id is disabled
        if not vnic_config.get('auto_slot_id', True):
            placement['Id'] = vnic_config['placement_slot_id']

        # Add PCI link configuration if auto_pci_link is disabled
        if not vnic_config.get('auto_pci_link', True):
            placement['PciLinkAssignmentMode'] = vnic_config['pci_link_assignment_mode']
            if vnic_config['pci_link_assignment_mode'] == 'Custom':
                placement['PciLink'] = vnic_config.get('pci_link', 0)

        vnic_api_body['Placement'] = placement
        vnic_api_body['Order'] = vnic_config.get('order', 0)

    # Add common CDN configuration
    vnic_api_body['Cdn'] = build_cdn_config(vnic_config)

    # Add static MAC address if using static type
    if mac_address_type == 'static':
        vnic_api_body['StaticMacAddress'] = vnic_config['static_mac_address']

    # Resolve and add policy MOIDs
    policy_moids = resolve_vnic_policy_moids(intersight, policy_cache, module, vnic_config, 'FIAttached')
    vnic_api_body.update(policy_moids)

    # Add common connection type settings
    organization_name = module.params['organization']
    connection_settings = build_connection_settings(intersight, policy_cache, module, vnic_config, organization_name)
    vnic_api_body.update(connection_settings)

    # Add pin group name if specified
    if vnic_config.get('pin_group_name'):
        vnic_api_body['PinGroupName'] = vnic_config['pin_group_name']

    return vnic_api_body


def validate_input(module):
    """
    Validate module input parameters
    """
    # Validate FIAttached specific requirements
    target_platform = module.params.get('target_platform')
    if target_platform == 'FIAttached':
        validate_fi_attached_params(module)

    # Validate vNIC configurations
    vnics = module.params.get('vnics', [])
    for vnic_config in vnics:
        vnic_name = vnic_config.get('name')
        vnic_state = vnic_config.get('state', 'present')

        # Only validate present vNICs - absent vNICs only need name
        if vnic_state == 'present':
            # Validate common required fields
            required_fields = ['eth_qos_policy_name', 'eth_adapter_policy_name']
            for field in required_fields:
                if not vnic_config.get(field):
                    module.fail_json(msg=f"{field} is required when vNIC state is 'present' for vNIC '{vnic_name}'")

            # Validate target platform specific fields
            target_platform = module.params.get('target_platform')
            if target_platform == 'FIAttached':
                validate_fi_attached_vnic_config(module, vnic_config)
            else:
                validate_standalone_vnic_config(module, vnic_config)

            # Validate CDN configuration
            validate_cdn_config(module, vnic_config)

            connection_type = vnic_config.get('connection_type', 'none')

            # Validate connection type specific settings
            vnic_name = vnic_config.get('name', 'unknown')
            if connection_type == 'usnic':
                usnic_settings = vnic_config.get('usnic_settings')
                validate_usnic_settings(module, usnic_settings, vnic_name)
            elif connection_type == 'vmq':
                vmq_settings = vnic_config.get('vmq_settings', {})
                validate_vmq_settings(module, vmq_settings, vnic_name)
            elif connection_type == 'sriov':
                sriov_settings = vnic_config.get('sriov_settings', {})
                validate_sriov_settings(module, sriov_settings, vnic_name)


def main():
    # Define vNIC options
    vnic_options = dict(
        name=dict(type='str', required=True),
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        placement_slot_id=dict(type='str'),
        pci_link=dict(type='int', choices=[0, 1], default=0),
        uplink_port=dict(type='int', choices=[0, 1, 2, 3], default=0),
        order=dict(type='int', default=0),
        eth_network_policy_name=dict(type='str'),
        eth_qos_policy_name=dict(type='str'),
        eth_adapter_policy_name=dict(type='str'),
        mac_address_type=dict(type='str', choices=['pool', 'static'], default='pool'),
        static_mac_address=dict(type='str'),
        auto_slot_id=dict(type='bool', default=True),
        auto_pci_link=dict(type='bool', default=True),
        pci_link_assignment_mode=dict(type='str', choices=['Custom', 'Load-Balanced']),
        auto_vnic_placement_enabled=dict(type='bool', default=False),
        pin_group_name=dict(type='str'),
    )
    # Add connection settings argument specs
    vnic_options.update(get_common_settings_argument_spec())

    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        target_platform=dict(type='str', choices=['Standalone', 'FIAttached'], default='Standalone'),
        azure_qos_enabled=dict(type='bool', default=False),
        iqn_allocation_type=dict(type='str', choices=['None', 'Pool', 'Static'], default='None'),
        placement_mode=dict(type='str', choices=['custom', 'auto'], default='custom'),
        iqn_pool_name=dict(type='str'),
        static_iqn_name=dict(type='str'),
        vnics=dict(type='list', elements='dict', options=vnic_options)
    )

    required_if = [
        ['state', 'present', ['vnics']],
    ]

    module = AnsibleModule(
        argument_spec,
        required_if=required_if,
        supports_check_mode=True,
    )

    if module.params['state'] == 'present':
        validate_input(module)

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Resource path used to configure policy
    resource_path = '/vnic/LanConnectivityPolicies'
    # Define API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name']
    }

    if intersight.module.params['state'] == 'present':
        intersight.set_tags_and_description()

        target_platform = intersight.module.params['target_platform']
        api_target_platform = 'FIAttached' if target_platform == 'FIAttached' else 'Standalone'
        intersight.api_body['TargetPlatform'] = api_target_platform

        # Add FIAttached-specific parameters
        if intersight.module.params['target_platform'] == 'FIAttached':
            intersight.api_body['AzureQosEnabled'] = intersight.module.params['azure_qos_enabled']
            intersight.api_body['IqnAllocationType'] = intersight.module.params['iqn_allocation_type']
            intersight.api_body['PlacementMode'] = intersight.module.params['placement_mode']

            # Resolve IQN pool MOID if specified
            if intersight.module.params['iqn_pool_name']:
                iqn_pool_moid = intersight.get_moid_by_name_and_org(
                    resource_path='/iqnpool/Pools',
                    resource_name=intersight.module.params['iqn_pool_name'],
                    organization_name=intersight.module.params['organization']
                )
                if not iqn_pool_moid:
                    intersight.module.fail_json(msg=f"IQN Pool '{intersight.module.params['iqn_pool_name']}' not found in organization ' \
                    {intersight.module.params['organization']}'")
                intersight.api_body['IqnPool'] = iqn_pool_moid

            if intersight.module.params['static_iqn_name']:
                intersight.api_body['StaticIqnName'] = intersight.module.params['static_iqn_name']

    intersight.configure_policy_or_profile(resource_path=resource_path)

    # Save the LAN connectivity policy response
    lan_connectivity_policy_response = intersight.result['api_response']

    lan_connectivity_policy_moid = None
    if intersight.module.params['state'] == 'present' and lan_connectivity_policy_response:
        lan_connectivity_policy_moid = lan_connectivity_policy_response.get('Moid')

    # Process vNICs
    vnics_response = []
    if intersight.module.params['state'] == 'present' and intersight.module.params.get('vnics'):
        # Cache for policy MOIDs to avoid redundant API calls
        policy_cache = {}

        for vnic_config in intersight.module.params['vnics']:
            vnic_state = vnic_config.get('state', 'present')

            # Only build API body for present vNICs
            if vnic_state == 'present':
                # Build vNIC API body using helper function
                vnic_api_body = build_vnic_api_body(
                    intersight, policy_cache, module, vnic_config, lan_connectivity_policy_moid
                )
                intersight.api_body = vnic_api_body

            # Configure the vNIC (create/update/delete)
            resource_path = '/vnic/EthIfs'
            # Filter by both vNIC name AND LAN Connectivity Policy to avoid affecting vNICs in other policies
            custom_filter = f"Name eq '{vnic_config['name']}' and LanConnectivityPolicy.Moid eq '{lan_connectivity_policy_moid}'"
            intersight.configure_secondary_resource(
                resource_path=resource_path,
                state=vnic_state,
                custom_filter=custom_filter
            )

            # Save the vNIC response only if it's present
            if vnic_state == 'present':
                vnics_response.append(intersight.result['api_response'])

    # Combine LAN connectivity policy and vNICs in the main response
    if lan_connectivity_policy_response:
        lan_connectivity_policy_response['vNICs'] = vnics_response
        intersight.result['api_response'] = lan_connectivity_policy_response

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



# Copyright (c) 2020-2021, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: reverse_pointer
short_description: Convert an IP address into a DNS name for reverse lookup
version_added: 3.1.0
description:
  - Given an IPv4 or IPv6 address, such as V(192.168.1.2), converts it to a DNS name to use for reverse lookups, such as V(2.1.168.192.in-addr.arpa).
options:
  _input:
    description:
      - The IP address.
    type: string
    required: true
author:
  - Felix Fontein (@felixfontein)
seealso:
  - name: RFC 1035, Section 3.5
    link: https://www.rfc-editor.org/rfc/rfc1035.html#section-3.5
    description: Describes C(in-addr.arpa).
  - name: RFC 3152
    link: https://www.rfc-editor.org/rfc/rfc3152.html
    description: Describes C(ip6.arpa).
"""

EXAMPLES = r"""
- name: Convert IP address to DNS name for reverse lookup
  ansible.builtin.set_fact:
    dns_name: "{{ ip_address | community.dns.reverse_pointer }}"
  # Should result in '2.1.168.192.in-addr.arpa.'
  vars:
    ip_address: 192.168.1.2
"""

RETURN = r"""
_value:
  description: The DNS name.
  type: string
"""


import ipaddress
import typing as t
from collections.abc import Callable

from ansible.errors import AnsibleFilterError
from ansible.module_utils.common.text.converters import to_text


def reverse_pointer(ip: t.Any) -> str:
    if not isinstance(ip, (str, bytes)):
        raise AnsibleFilterError(
            "Input for community.dns.reverse_pointer must be a string"
        )
    try:
        ipaddr = ipaddress.ip_address(to_text(ip))
    except Exception as e:
        raise AnsibleFilterError(f"Cannot parse IP address: {e}") from e
    res = ipaddr.reverse_pointer
    if not res.endswith("."):
        res += "."
    else:
        pass  # pragma: no cover
    return res


class FilterModule:
    """Ansible jinja2 filters"""

    def filters(self) -> dict[str, Callable]:
        return {
            "reverse_pointer": reverse_pointer,
        }
